"""
How to delete slaves/nodes
"""
from __future__ import print_function
import logging
from jenkinsapi.jenkins import Jenkins
logging.basicConfig()


j = Jenkins('http://localhost:8080')

for node_id, _ in j.get_nodes().iteritems():
    if node_id != 'master':
        print(node_id)
        j.delete_node(node_id)

# Alternative way - this method will not delete 'master'
for node in j.nodes.keys():
    del j.nodes[node]



# Code from https://github.com/ros-infrastructure/ros_buildfarm
# (c) Open Source Robotics Foundation
import ast
import logging
from jenkinsapi.utils.requester import Requester

logger = logging.getLogger(__name__)


class CrumbRequester(Requester):

    """Adapter for Requester inserting the crumb in every request."""

    def __init__(self, *args, **kwargs):
        super(CrumbRequester, self).__init__(*args, **kwargs)
        self._baseurl = kwargs['baseurl']
        self._last_crumb_data = None

    def post_url(self, url, params=None, data=None, files=None,
                 headers=None, allow_redirects=True, **kwargs):
        if self._last_crumb_data:
            # first try request with previous crumb if available
            response = self._post_url_with_crumb(
                self._last_crumb_data, url, params, data,
                files, headers, allow_redirects, **kwargs
            )
            # code 403 might indicate that the crumb is not valid anymore
            if response.status_code != 403:
                return response

        # fetch new crumb (if server has crumbs enabled)
        if self._last_crumb_data is not False:
            self._last_crumb_data = self._get_crumb_data()

        return self._post_url_with_crumb(
            self._last_crumb_data, url, params, data,
            files, headers, allow_redirects, **kwargs)

    def _get_crumb_data(self):
        response = self.get_url(self._baseurl + '/crumbIssuer/api/python')
        if response.status_code in [404]:
            logger.warning('The Jenkins master does not require a crumb')
            return False
        if response.status_code not in [200]:
            raise RuntimeError('Failed to fetch crumb: %s' % response.text)
        crumb_issuer_response = ast.literal_eval(response.text)
        crumb_request_field = crumb_issuer_response['crumbRequestField']
        crumb = crumb_issuer_response['crumb']
        logger.debug('Fetched crumb: %s', crumb)
        return {crumb_request_field: crumb}

    def _post_url_with_crumb(self, crumb_data, url, params, data,
                             files, headers, allow_redirects, **kwargs):
        if crumb_data:
            if headers is None:
                headers = crumb_data
            else:
                headers.update(crumb_data)

        return super(CrumbRequester, self).post_url(
            url, params, data, files, headers, allow_redirects, **kwargs)



"""
System tests for `jenkinsapi.jenkins` module.
"""
import logging
import pytest
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi.credentials import Credentials
from jenkinsapi.credentials import UsernamePasswordCredential
from jenkinsapi.credentials import SecretTextCredential
from jenkinsapi.credential import SSHKeyCredential

log = logging.getLogger(__name__)


def test_get_credentials(jenkins):
    creds = jenkins.credentials
    assert isinstance(creds, Credentials) is True


def test_delete_inexistant_credential(jenkins):
    with pytest.raises(KeyError):
        creds = jenkins.credentials

        del creds[random_string()]


def test_create_user_pass_credential(jenkins):
    creds = jenkins.credentials

    cred_descr = random_string()
    cred_dict = {
        'description': cred_descr,
        'userName': 'userName',
        'password': 'password'
    }
    creds[cred_descr] = UsernamePasswordCredential(cred_dict)

    assert cred_descr in creds

    cred = creds[cred_descr]
    assert isinstance(cred, UsernamePasswordCredential) is True
    assert cred.password is None
    assert cred.description == cred_descr

    del creds[cred_descr]


def test_update_user_pass_credential(jenkins):
    creds = jenkins.credentials

    cred_descr = random_string()
    cred_dict = {
        'description': cred_descr,
        'userName': 'userName',
        'password': 'password'
    }
    creds[cred_descr] = UsernamePasswordCredential(cred_dict)

    cred = creds[cred_descr]
    cred.userName = 'anotheruser'
    cred.password = 'password2'

    cred = creds[cred_descr]
    assert isinstance(cred, UsernamePasswordCredential) is True
    assert cred.userName == 'anotheruser'
    assert cred.password == 'password2'


def test_create_ssh_credential(jenkins):
    creds = jenkins.credentials

    cred_descr = random_string()
    cred_dict = {
        'description': cred_descr,
        'userName': 'userName',
        'passphrase': '',
        'private_key': '-----BEGIN RSA PRIVATE KEY-----'
    }
    creds[cred_descr] = SSHKeyCredential(cred_dict)

    assert cred_descr in creds

    cred = creds[cred_descr]
    assert isinstance(cred, SSHKeyCredential) is True
    assert cred.description == cred_descr

    del creds[cred_descr]

    cred_dict = {
        'description': cred_descr,
        'userName': 'userName',
        'passphrase': '',
        'private_key': '/tmp/key'
    }
    with pytest.raises(ValueError):
        creds[cred_descr] = SSHKeyCredential(cred_dict)

    cred_dict = {
        'description': cred_descr,
        'userName': 'userName',
        'passphrase': '',
        'private_key': '~/.ssh/key'
    }
    with pytest.raises(ValueError):
        creds[cred_descr] = SSHKeyCredential(cred_dict)

    cred_dict = {
        'description': cred_descr,
        'userName': 'userName',
        'passphrase': '',
        'private_key': 'invalid'
    }
    with pytest.raises(ValueError):
        creds[cred_descr] = SSHKeyCredential(cred_dict)


def test_delete_credential(jenkins):
    creds = jenkins.credentials

    cred_descr = random_string()
    cred_dict = {
        'description': cred_descr,
        'userName': 'userName',
        'password': 'password'
    }
    creds[cred_descr] = UsernamePasswordCredential(cred_dict)

    assert cred_descr in creds
    del creds[cred_descr]
    assert cred_descr not in creds


def test_create_secret_text_credential(jenkins):
    """
    Tests the creation of a secret text.
    """
    creds = jenkins.credentials

    cred_descr = random_string()
    cred_dict = {
        'description': cred_descr,
        'secret': 'newsecret'
    }
    creds[cred_descr] = SecretTextCredential(cred_dict)

    assert cred_descr in creds
    cred = creds[cred_descr]
    assert isinstance(cred, SecretTextCredential) is True
    assert cred.secret is None
    assert cred.description == cred_descr

    del creds[cred_descr]



"""
System tests for `jenkinsapi.jenkins` module.
"""
import re
import time
from jenkinsapi_tests.systests.job_configs import MATRIX_JOB
from jenkinsapi_tests.test_utils.random_strings import random_string


def test_invoke_matrix_job(jenkins):
    job_name = 'create_%s' % random_string()
    job = jenkins.create_job(job_name, MATRIX_JOB)
    queueItem = job.invoke()
    queueItem.block_until_complete()

    build = job.get_last_build()

    while build.is_running():
        time.sleep(1)

    set_of_groups = set()
    for run in build.get_matrix_runs():
        assert run.get_number() == build.get_number()
        assert run.get_upstream_build() == build
        match_result = re.search(u'\xbb (.*) #\\d+$', run.name)
        assert match_result is not None
        set_of_groups.add(match_result.group(1))
        build.get_master_job_name()

    assert set_of_groups == set(['one', 'two', 'three'])



import os
import logging


logging.basicConfig(
    format='%(module)s.%(funcName)s %(levelname)s: %(message)s',
    level=logging.INFO
)

level = logging.WARNING if 'LOG_LEVEL' not in os.environ \
    else os.environ['LOG_LEVEL'].upper().strip()

modules = [
    'requests.packages.urllib3.connectionpool',
    'requests',
    'urllib3',
    'urllib3.connectionpool'
]

for module_name in modules:
    logger = logging.getLogger(module_name)
    logger.setLevel(level)



import time
import json
import logging
import pytest

from six import StringIO
from six.moves.urllib.parse import urljoin
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.utils.crumb_requester import CrumbRequester
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi_tests.systests.job_configs import JOB_WITH_FILE


log = logging.getLogger(__name__)

DEFAULT_JENKINS_PORT = 8080

ENABLE_CRUMBS_CONFIG = {
    'hudson-security-csrf-GlobalCrumbIssuerConfiguration': {
        'csrf': {
            'issuer': {
                'value': '0',
                'stapler-class': 'hudson.security.csrf.DefaultCrumbIssuer',
                '$class': 'hudson.security.csrf.DefaultCrumbIssuer',
                'excludeClientIPFromCrumb': False
            }
        }
    }
}

DISABLE_CRUMBS_CONFIG = {
    'hudson-security-csrf-GlobalCrumbIssuerConfiguration': {},
}

SECURITY_SETTINGS = {
    '': '0',
    'markupFormatter': {
        'stapler-class': 'hudson.markup.EscapedMarkupFormatter',
        '$class': 'hudson.markup.EscapedMarkupFormatter'
    },
    'org-jenkinsci-main-modules-sshd-SSHD': {
        'port': {
            'value': '',
            'type': 'disabled'
        }
    },
    'jenkins-CLI': {
        'enabled': False
    },
    # This is not required if envinject plugin is not installed
    # but since it is installed for test suite - we must have this config
    # If this is not present - Jenkins will return error
    'org-jenkinsci-plugins-envinject-EnvInjectPluginConfiguration': {
        'enablePermissions': False,
        'hideInjectedVars': False,
        'enableLoadingFromMaster': False
    },
    'jenkins-model-DownloadSettings': {
        'useBrowser': False
    },
    'slaveAgentPort': {
        'value': '',
        'type': 'disable'
    },
    'agentProtocol': [
        'CLI-connect',
        'CLI2-connect',
        'JNLP-connect',
        'JNLP2-connect',
        'JNLP4-connect'
    ],
    'core:apply': ''
}


@pytest.fixture(scope='function')
def crumbed_jenkins(jenkins):
    ENABLE_CRUMBS_CONFIG.update(SECURITY_SETTINGS)
    DISABLE_CRUMBS_CONFIG.update(SECURITY_SETTINGS)

    jenkins.requester.post_and_confirm_status(
        urljoin(jenkins.baseurl, '/configureSecurity/configure'),
        data={
            'Submit': 'save',
            'json': json.dumps(ENABLE_CRUMBS_CONFIG)
        },
        headers={'Content-Type': 'application/x-www-form-urlencoded'}
    )
    log.info('Enabled Jenkins security')

    crumbed = Jenkins(
        jenkins.baseurl,
        requester=CrumbRequester(baseurl=jenkins.baseurl)
    )

    yield crumbed

    crumbed.requester.post_and_confirm_status(
        jenkins.baseurl + '/configureSecurity/configure',
        data={
            'Submit': 'save',
            'json': json.dumps(DISABLE_CRUMBS_CONFIG)
        },
        headers={'Content-Type': 'application/x-www-form-urlencoded'}
    )
    log.info('Disabled Jenkins security')


def test_invoke_job_with_file(crumbed_jenkins):
    file_data = random_string()
    param_file = StringIO(file_data)

    job_name = 'create1_%s' % random_string()
    job = crumbed_jenkins.create_job(job_name, JOB_WITH_FILE)

    assert job.has_params()
    assert len(job.get_params_list())

    job.invoke(block=True, files={'file.txt': param_file})

    build = job.get_last_build()
    while build.is_running():
        time.sleep(0.25)

    artifacts = build.get_artifact_dict()
    assert isinstance(artifacts, dict) is True
    art_file = artifacts['file.txt']
    assert art_file.get_data().decode('utf-8').strip() == file_data




"""
System tests for setting jenkins in quietDown mode
"""
import pytest
import logging


log = logging.getLogger(__name__)


@pytest.mark.run_these_please
def test_quiet_down_and_cancel_quiet_down(jenkins):
    jenkins.poll()  # jenkins should be alive

    jenkins.quiet_down()  # put Jenkins in quietDown mode
    # is_quieting_down = jenkins.is_quieting_down
    assert jenkins.is_quieting_down is True

    jenkins.poll()  # jenkins should be alive

    jenkins.cancel_quiet_down()  # leave quietDown mode

    # is_quieting_down = jenkins_api['quietingDown']
    assert jenkins.is_quieting_down is False

    jenkins.poll()  # jenkins should be alive



"""
All kinds of testing on Jenkins Queues
"""
import time
import logging
import pytest
from jenkinsapi.queue import Queue
from jenkinsapi.queue import QueueItem
from jenkinsapi.job import Job
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi_tests.systests.job_configs import LONG_RUNNING_JOB

log = logging.getLogger(__name__)


@pytest.fixture(scope='function')
def no_executors(jenkins, request):
    master = jenkins.nodes['master']
    num_executors = master.get_num_executors()
    master.set_num_executors(0)

    def restore():
        master.set_num_executors(num_executors)

    request.addfinalizer(restore)

    return num_executors


def test_get_queue(jenkins):
    qq = jenkins.get_queue()
    assert isinstance(qq, Queue) is True


def test_invoke_many_jobs(jenkins, no_executors):
    job_names = [random_string() for _ in range(5)]
    jobs = []

    while len(jenkins.get_queue()) != 0:
        log.info('Sleeping to get queue empty...')
        time.sleep(1)

    for job_name in job_names:
        j = jenkins.create_job(job_name, LONG_RUNNING_JOB)
        jobs.append(j)
        j.invoke()

        assert j.is_queued_or_running() is True

    queue = jenkins.get_queue()

    reprString = repr(queue)
    assert queue.baseurl in reprString
    assert len(queue) == 5, queue.keys()
    assert isinstance(queue[queue.keys()[0]].get_job(), Job) is True
    items = queue.get_queue_items_for_job(job_names[2])
    assert isinstance(items, list) is True
    assert len(items) == 1
    assert isinstance(items[0], QueueItem) is True
    assert items[0].get_parameters() == []

    for _, item in queue.iteritems():
        queue.delete_item(item)

    queue.poll()

    assert len(queue) == 0


def test_start_and_stop_long_running_job(jenkins):
    job_name = random_string()
    j = jenkins.create_job(job_name, LONG_RUNNING_JOB)
    j.invoke()
    time.sleep(1)
    assert j.is_queued_or_running() is True

    while j.is_queued():
        time.sleep(0.5)

    if j.is_running():
        time.sleep(1)

    j.get_first_build().stop()
    time.sleep(1)
    assert j.is_queued_or_running() is False


def test_queueitem_for_why_field(jenkins, no_executors):
    job_names = [random_string() for _ in range(2)]

    jobs = []
    for job_name in job_names:
        j = jenkins.create_job(job_name, LONG_RUNNING_JOB)
        jobs.append(j)
        j.invoke()

    queue = jenkins.get_queue()
    for _, item in queue.iteritems():
        assert isinstance(item.why, str) is True

    # Clean up after ourselves
    for _, item in queue.iteritems():
        queue.delete_item(item)


def test_queueitem_from_job(jenkins, no_executors):
    job_name = random_string()
    j = jenkins.create_job(job_name, LONG_RUNNING_JOB)
    j.invoke()

    qi = j.get_queue_item()
    assert isinstance(qi, QueueItem)
    assert qi.get_job() == j
    assert qi.get_job_name() == job_name
    assert qi.name == job_name
    assert qi.is_queued()
    assert not qi.is_running()



import pytest
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.nodes import Nodes
from jenkinsapi.node import Node


DATA0 = {
    'assignedLabels': [{}],
    'description': None,
    'jobs': [],
    'mode': 'NORMAL',
    'nodeDescription': 'the master Jenkins node',
    'nodeName': '',
    'numExecutors': 2,
    'overallLoad': {},
    'primaryView': {'name': 'All', 'url': 'http://halob:8080/'},
    'quietingDown': False,
    'slaveAgentPort': 0,
    'unlabeledLoad': {},
    'useCrumbs': False,
    'useSecurity': False,
    'views': [
        {'name': 'All', 'url': 'http://halob:8080/'},
        {'name': 'FodFanFo', 'url': 'http://halob:8080/view/FodFanFo/'}
    ]
}

DATA1 = {
    'busyExecutors': 0,
    'computer': [
        {
            'actions': [],
            'displayName': 'master',
            'executors': [{}, {}],
            'icon': 'computer.png',
            'idle': True,
            'jnlpAgent': False,
            'launchSupported': True,
            'loadStatistics': {},
            'manualLaunchAllowed': True,
            'monitorData': {
                'hudson.node_monitors.ArchitectureMonitor': 'Linux (amd64)',
                'hudson.node_monitors.ClockMonitor': {'diff': 0},
                'hudson.node_monitors.DiskSpaceMonitor': {
                    'path': '/var/lib/jenkins',
                    'size': 671924924416
                },
                'hudson.node_monitors.ResponseTimeMonitor': {'average': 0},
                'hudson.node_monitors.SwapSpaceMonitor': {
                    'availablePhysicalMemory': 3174686720,
                    'availableSwapSpace': 17163087872,
                    'totalPhysicalMemory': 16810180608,
                    'totalSwapSpace': 17163087872
                },
                'hudson.node_monitors.TemporarySpaceMonitor': {
                    'path': '/tmp',
                    'size': 671924924416
                }
            },
            'numExecutors': 2,
            'offline': False,
            'offlineCause': None,
            'oneOffExecutors': [],
            'temporarilyOffline': False
        },
        {
            'actions': [],
            'displayName': 'bobnit',
            'executors': [{}],
            'icon': 'computer-x.png',
            'idle': True,
            'jnlpAgent': False,
            'launchSupported': True,
            'loadStatistics': {},
            'manualLaunchAllowed': True,
            'monitorData': {
                'hudson.node_monitors.ArchitectureMonitor': 'Linux (amd64)',
                'hudson.node_monitors.ClockMonitor': {'diff': 4261},
                'hudson.node_monitors.DiskSpaceMonitor': {
                    'path': '/home/sal/jenkins',
                    'size': 169784860672
                },
                'hudson.node_monitors.ResponseTimeMonitor': {'average': 29},
                'hudson.node_monitors.SwapSpaceMonitor': {
                    'availablePhysicalMemory': 4570710016,
                    'availableSwapSpace': 12195983360,
                    'totalPhysicalMemory': 8374497280,
                    'totalSwapSpace': 12195983360
                },
                'hudson.node_monitors.TemporarySpaceMonitor': {
                    'path': '/tmp',
                    'size': 249737277440
                }
            },
            'numExecutors': 1,
            'offline': True,
            'offlineCause': {},
            'oneOffExecutors': [],
            'temporarilyOffline': False
        },
        {
            'actions': [],
            'displayName': 'halob',
            'executors': [{}],
            'icon': 'computer-x.png',
            'idle': True,
            'jnlpAgent': True,
            'launchSupported': False,
            'loadStatistics': {},
            'manualLaunchAllowed': True,
            'monitorData': {
                'hudson.node_monitors.ArchitectureMonitor': None,
                'hudson.node_monitors.ClockMonitor': None,
                'hudson.node_monitors.DiskSpaceMonitor': None,
                'hudson.node_monitors.ResponseTimeMonitor': None,
                'hudson.node_monitors.SwapSpaceMonitor': None,
                'hudson.node_monitors.TemporarySpaceMonitor': None
            },
            'numExecutors': 1,
            'offline': True,
            'offlineCause': None,
            'oneOffExecutors': [],
            'temporarilyOffline': False
        }
    ],
    'displayName': 'nodes',
    'totalExecutors': 2
}

DATA2 = {
    'actions': [],
    'displayName': 'master',
    'executors': [{}, {}],
    'icon': 'computer.png',
    'idle': True,
    'jnlpAgent': False,
    'launchSupported': True,
    'loadStatistics': {},
    'manualLaunchAllowed': True,
    'monitorData': {
        'hudson.node_monitors.ArchitectureMonitor': 'Linux (amd64)',
        'hudson.node_monitors.ClockMonitor': {'diff': 0},
        'hudson.node_monitors.DiskSpaceMonitor': {
            'path': '/var/lib/jenkins',
            'size': 671942561792
        },
        'hudson.node_monitors.ResponseTimeMonitor': {'average': 0},
        'hudson.node_monitors.SwapSpaceMonitor': {
            'availablePhysicalMemory': 2989916160,
            'availableSwapSpace': 17163087872,
            'totalPhysicalMemory': 16810180608,
            'totalSwapSpace': 17163087872
        },
        'hudson.node_monitors.TemporarySpaceMonitor': {
            'path': '/tmp',
            'size': 671942561792
        }
    },
    'numExecutors': 2,
    'offline': False,
    'offlineCause': None,
    'oneOffExecutors': [],
    'temporarilyOffline': False
}

DATA3 = {
    'actions': [],
    'displayName': 'halob',
    'executors': [{}],
    'icon': 'computer-x.png',
    'idle': True,
    'jnlpAgent': True,
    'launchSupported': False,
    'loadStatistics': {},
    'manualLaunchAllowed': True,
    'monitorData': {
        'hudson.node_monitors.ArchitectureMonitor': None,
        'hudson.node_monitors.ClockMonitor': None,
        'hudson.node_monitors.DiskSpaceMonitor': None,
        'hudson.node_monitors.ResponseTimeMonitor': None,
        'hudson.node_monitors.SwapSpaceMonitor': None,
        'hudson.node_monitors.TemporarySpaceMonitor': None},
    'numExecutors': 1,
    'offline': True,
    'offlineCause': None,
    'oneOffExecutors': [],
    'temporarilyOffline': False
}


@pytest.fixture(scope='function')
def nodes(monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        return DATA0

    monkeypatch.setattr(Jenkins, '_poll', fake_jenkins_poll)

    def fake_nodes_poll(cls, tree=None):  # pylint: disable=unused-argument
        return DATA1

    monkeypatch.setattr(Nodes, '_poll', fake_nodes_poll)
    jenkins = Jenkins('http://foo:8080')
    return jenkins.get_nodes()


def fake_node_poll(self, tree=None):  # pylint: disable=unused-argument
    """
    Fakes a poll of data by returning the correct section of the DATA1 test block.
    """
    for node_poll in DATA1['computer']:
        if node_poll['displayName'] == self.name:
            return node_poll
    return DATA2


def test_repr(nodes):
    # Can we produce a repr string for this object
    repr(nodes)


def test_baseurl(nodes):
    assert nodes.baseurl == 'http://foo:8080/computer'


def test_get_master_node(nodes, monkeypatch):
    monkeypatch.setattr(Node, '_poll', fake_node_poll)

    node = nodes['master']
    assert isinstance(node, Node)


def test_get_nonmaster_node(nodes, monkeypatch):
    monkeypatch.setattr(Node, '_poll', fake_node_poll)

    node = nodes['halob']
    assert isinstance(node, Node)


def test_iterkeys(nodes):
    expected_names = set(['master', 'bobnit', 'halob'])
    actual_names = set([n for n in nodes.iterkeys()])
    assert actual_names == expected_names


def test_keys(nodes):
    expected_names = set(['master', 'bobnit', 'halob'])
    actual_names = set(nodes.keys())
    assert actual_names == expected_names


def items_test_case(nodes_method, monkeypatch):
    monkeypatch.setattr(Node, '_poll', fake_node_poll)

    expected_names = set(['master', 'bobnit', 'halob'])

    actual_names = set()
    for name, node in nodes_method():
        assert name == node.name
        assert isinstance(node, Node)

        actual_names.add(name)

    assert actual_names == expected_names


def test_iteritems(nodes, monkeypatch):
    items_test_case(nodes.iteritems, monkeypatch)


def test_items(nodes, monkeypatch):
    items_test_case(nodes.items, monkeypatch)


def values_test_case(nodes_method, monkeypatch):
    monkeypatch.setattr(Node, '_poll', fake_node_poll)

    expected_names = set(['master', 'bobnit', 'halob'])

    actual_names = set()
    for node in nodes_method():
        assert isinstance(node, Node)
        actual_names.add(node.name)

    assert actual_names == expected_names


def test_itervalues(nodes, monkeypatch):
    values_test_case(nodes.itervalues, monkeypatch)


def test_values(nodes, monkeypatch):
    values_test_case(nodes.values, monkeypatch)



import requests
import warnings
import pytest
import pytz
from . import configs
import datetime
from jenkinsapi.build import Build
from jenkinsapi.job import Job


@pytest.fixture(scope='function')
def jenkins(mocker):
    return mocker.MagicMock()


@pytest.fixture(scope='function')
def job(monkeypatch, jenkins):
    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return configs.JOB_DATA

    monkeypatch.setattr(Job, '_poll', fake_poll)

    fake_job = Job('http://', 'Fake_Job', jenkins)
    return fake_job


@pytest.fixture(scope='function')
def build(job, monkeypatch):
    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return configs.BUILD_DATA

    monkeypatch.setattr(Build, '_poll', fake_poll)

    return Build('http://', 97, job)


@pytest.fixture(scope='function')
def build_pipeline(job, monkeypatch):
    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return configs.BUILD_DATA_PIPELINE

    monkeypatch.setattr(Build, '_poll', fake_poll)

    return Build('http://', 97, job)


def test_timestamp(build):
    assert isinstance(build.get_timestamp(), datetime.datetime)

    expected = pytz.utc.localize(
        datetime.datetime(2013, 5, 31, 23, 15, 40))

    assert build.get_timestamp() == expected


def test_name(build):
    with pytest.raises(AttributeError):
        build.id()
    assert build.name == 'foo #1'


def test_duration(build):
    expected = datetime.timedelta(milliseconds=5782)
    assert build.get_duration() == expected
    assert build.get_duration().seconds == 5
    assert build.get_duration().microseconds == 782000
    assert str(build.get_duration()) == '0:00:05.782000'


def test_get_causes(build):
    assert build.get_causes() == [{
        'shortDescription': 'Started by user anonymous',
        'userId': None,
        'userName': 'anonymous'
    }]


def test_get_changeset(build):
    assert build.get_changeset_items() == [{
        "affectedPaths": [
            "content/rcm/v00-rcm-xccdf.xml"
        ],
        "author": {
            "absoluteUrl": "http://jenkins_url/user/username79",
            "fullName": "username"
        },
        "commitId": "3097",
        "timestamp": 1414398423091,
        "date": "2014-10-27T08:27:03.091288Z",
        "msg": "commit message",
        "paths": [{
            "editType": "edit",
            "file": "/some/path/of/changed_file"
        }],
        "revision": 3097,
        "user": "username"
        }]


def test_get_changeset_pipeline(build_pipeline):
    assert build_pipeline.get_changeset_items() == [{
        "affectedPaths": [
            "content/rcm/v00-rcm-xccdf.xml"
        ],
        "author": {
            "absoluteUrl": "http://jenkins_url/user/username79",
            "fullName": "username"
        },
        "commitId": "3097",
        "timestamp": 1414398423091,
        "date": "2014-10-27T08:27:03.091288Z",
        "msg": "commit message",
        "paths": [{
            "editType": "edit",
            "file": "/some/path/of/changed_file"
        }],
        "revision": 3097,
        "user": "username"
    }]


def test_get_description(build):
    assert build.get_description() == 'Best build ever!'


def test_get_slave(build):
    assert build.get_slave() == 'localhost'


def test_get_revision_no_scm(build):
    """ with no scm, get_revision should return None """
    assert build.get_revision() is None


def test_downstream(build):
    expected = ['test1', 'test2']
    assert build.get_downstream_job_names() == expected


def test_get_params(build):
    expected = {
        'first_param': 'first_value',
        'second_param': 'second_value',
    }
    build._data = {
        'actions': [{
            '_class': 'hudson.model.ParametersAction',
            'parameters': [
                {'name': 'first_param', 'value': 'first_value'},
                {'name': 'second_param', 'value': 'second_value'},
            ]
        }]
    }
    params = build.get_params()
    assert params == expected


def test_get_build_url(build):
    expected = "http://foo/1"
    build._data = {"url": "http://foo/1"}
    url = build.get_build_url()
    assert url == expected


def test_get_params_different_order(build):
    """
    Dictionary with `parameters` key is not always the first element in
    `actions` list, so we need to search through whole array. This test
    covers such a case
    """
    expected = {
        'first_param': 'first_value',
        'second_param': 'second_value',
    }
    build._data = {
        'actions': [
            {
                'not_parameters': 'some_data',
            },
            {
                'another_action': 'some_value',
            },
            {
                '_class': 'hudson.model.ParametersAction',
                'parameters': [
                    {'name': 'first_param', 'value': 'first_value'},
                    {'name': 'second_param', 'value': 'second_value'},
                ]
            }
        ]
    }
    params = build.get_params()
    assert params == expected


def test_only_ParametersAction_parameters_considered(build):
    """Actions other than ParametersAction can have dicts called parameters."""
    expected = {
        'param': 'value',
    }
    build._data = {
        'actions': [
            {
                '_class': 'hudson.model.SomeOtherAction',
                'parameters': [
                    {'name': 'Not', 'value': 'OurValue'},
                ]
            },
            {
                '_class': 'hudson.model.ParametersAction',
                'parameters': [
                    {'name': 'param', 'value': 'value'},
                ]
            }
        ]
    }
    params = build.get_params()
    assert params == expected


def test_ParametersWithNoValueSetValueNone_issue_583(build):
    """SecretParameters don't share their value in the API."""
    expected = {
        'some-secret': None,
    }
    build._data = {
        'actions': [
            {
                '_class': 'hudson.model.ParametersAction',
                'parameters': [
                    {'name': 'some-secret'},
                ]
            }
        ]
    }
    params = build.get_params()
    assert params == expected


def test_build_env_vars(monkeypatch, build):
    def fake_get_data(cls, tree=None, params=None):
        return configs.BUILD_ENV_VARS
    monkeypatch.setattr(Build, 'get_data', fake_get_data)
    assert build.get_env_vars() == configs.BUILD_ENV_VARS['envMap']


def test_build_env_vars_wo_injected_env_vars_plugin(monkeypatch, build):
    def fake_get_data(cls, tree=None, params=None):
        raise requests.HTTPError('404')
    monkeypatch.setattr(Build, 'get_data', fake_get_data)

    with pytest.raises(requests.HTTPError) as excinfo:
        with pytest.warns(None) as record:
            build.get_env_vars()
    assert '404' == str(excinfo.value)
    assert len(record) == 1
    expected = UserWarning('Make sure the Environment Injector '
                           'plugin is installed.')
    assert str(record[0].message) == str(expected)


def test_build_env_vars_other_exception(monkeypatch, build):
    def fake_get_data(cls, tree=None, params=None):
        raise ValueError()
    monkeypatch.setattr(Build, 'get_data', fake_get_data)

    with pytest.raises(Exception) as excinfo:
        with pytest.warns(None) as record:
            build.get_env_vars()
    assert '' == str(excinfo.value)
    assert len(record) == 0



from jenkinsapi import config

JOB_DATA = {
    "actions": [{
        "parameterDefinitions": [{
            "defaultParameterValue": {
                "name": "param1",
                "value": "test1"
            },
            "description": "",
            "name": "param1",
            "type": "StringParameterDefinition"
        }, {
            "defaultParameterValue": {
                "name": "param2",
                "value": ""
            },
            "description": "",
            "name": "param2",
            "type": "StringParameterDefinition"
        }]
    }],
    "description": "test job",
    "displayName": "foo",
    "displayNameOrNull": None,
    "name": "foo",
    "url": "http://halob:8080/job/foo/",
    "buildable": True,
    "builds": [
        {"number": 3, "url": "http://halob:8080/job/foo/3/"},
        {"number": 2, "url": "http://halob:8080/job/foo/2/"},
        {"number": 1, "url": "http://halob:8080/job/foo/1/"}
    ],
    # allBuilds is not present in job dict returned by Jenkins
    # it is inserted here to test _add_missing_builds()
    "allBuilds": [
        {"number": 3, "url": "http://halob:8080/job/foo/3/"},
        {"number": 2, "url": "http://halob:8080/job/foo/2/"},
        {"number": 1, "url": "http://halob:8080/job/foo/1/"}
    ],
    "color": "blue",
    "firstBuild": {"number": 1, "url": "http://halob:8080/job/foo/1/"},
    "healthReport": [{
        "description": "Build stability: No recent builds failed.",
        "iconUrl": "health-80plus.png", "score": 100
    }],
    "inQueue": False,
    "keepDependencies": False,
    # build running
    "lastBuild": {"number": 4, "url": "http://halob:8080/job/foo/4/"},
    "lastCompletedBuild": {"number": 3,
                           "url": "http://halob:8080/job/foo/3/"},
    "lastFailedBuild": None,
    "lastStableBuild": {"number": 3,
                        "url": "http://halob:8080/job/foo/3/"},
    "lastSuccessfulBuild": {"number": 3,
                            "url": "http://halob:8080/job/foo/3/"},
    "lastUnstableBuild": None,
    "lastUnsuccessfulBuild": None,
    "nextBuildNumber": 4,
    "property": [],
    "queueItem": None,
    "concurrentBuild": False,
    # test1 job exists, test2 does not
    "downstreamProjects": [{'name': 'test1'}, {'name': 'test2'}],
    "scm": {},
    "upstreamProjects": []
}

URL_DATA = {'http://halob:8080/job/foo/%s' % config.JENKINS_API: JOB_DATA}

BUILD_DATA = {
    'actions': [{
        'causes': [{
            'shortDescription': 'Started by user anonymous',
            'userId': None,
            'userName': 'anonymous'
        }]
    }],
    'artifacts': [],
    'building': False,
    'builtOn': 'localhost',
    'changeSet': {'items': [{
        "affectedPaths": [
            "content/rcm/v00-rcm-xccdf.xml"
        ],
        "author": {
            "absoluteUrl": "http://jenkins_url/user/username79",
            "fullName": "username"
        },
        "commitId": "3097",
        "timestamp": 1414398423091,
        "date": "2014-10-27T08:27:03.091288Z",
        "msg": "commit message",
        "paths": [{
            "editType": "edit",
            "file": "/some/path/of/changed_file"
        }],
        "revision": 3097,
        "user": "username"
        }], 'kind': None},
    'culprits': [],
    'description': 'Best build ever!',
    "duration": 5782,
    'estimatedDuration': 106,
    'executor': None,
    "fingerprint": [{
        "fileName": "BuildId.json",
        "hash": "e3850a45ab64aa34c1aa66e30c1a8977",
        "original": {"name": "ArtifactGenerateJob", "number": 469},
        "timestamp": 1380270162488,
        "usage": [{
            "name": "test1",
            "ranges": {
                "ranges": [{"end": 567, "start": 566}]
            }
        }, {
            "name": "test2",
            "ranges": {
                "ranges": [{"end": 150, "start": 139}]
            }
        }]
    }],
    'fullDisplayName': 'foo #1',
    'id': '2013-05-31_23-15-40',
    'keepLog': False,
    'number': 1,
    'result': 'SUCCESS',
    'timestamp': 1370042140000,
    'url': 'http://localhost:8080/job/foo/1/',
    'runs': [{
        'number': 1,
        'url': 'http//localhost:8080/job/foo/SHARD_NUM=1/1/'
    }, {
        'number': 2,
        'url': 'http//localhost:8080/job/foo/SHARD_NUM=1/2/'
    }]
}

BUILD_DATA_PIPELINE = {
    'actions': [{
        'causes': [{
            'shortDescription': 'Started by user anonymous',
            'userId': None,
            'userName': 'anonymous'
        }]
    }],
    'artifacts': [],
    'building': False,
    'builtOn': 'localhost',
    'changeSets': {
        'items': [{
            "affectedPaths": [
                "content/rcm/v00-rcm-xccdf.xml"
            ],
            "author": {
                "absoluteUrl": "http://jenkins_url/user/username79",
                "fullName": "username"
            },
            "commitId": "3097",
            "timestamp": 1414398423091,
            "date": "2014-10-27T08:27:03.091288Z",
            "msg": "commit message",
            "paths": [{
                "editType": "edit",
                "file": "/some/path/of/changed_file"
            }],
            "revision": 3097,
            "user": "username"
        }],
        'kind': None
    },
    'culprits': [],
    'description': 'Best build ever!',
    "duration": 5782,
    'estimatedDuration': 106,
    'executor': None,
    "fingerprint": [{
        "fileName": "BuildId.json",
        "hash": "e3850a45ab64aa34c1aa66e30c1a8977",
        "original": {"name": "ArtifactGenerateJob", "number": 469},
        "timestamp": 1380270162488,
        "usage": [{
            "name": "test1",
            "ranges": {
                "ranges": [{"end": 567, "start": 566}]
            }
        }, {
            "name": "test2",
            "ranges": {
                "ranges": [{"end": 150, "start": 139}]
            }
        }]
    }],
    'fullDisplayName': 'foo #1',
    'id': '2013-05-31_23-15-40',
    'keepLog': False,
    'number': 1,
    'result': 'SUCCESS',
    'timestamp': 1370042140000,
    'url': 'http://localhost:8080/job/foo/1/',
    'runs': [{
        'number': 1,
        'url': 'http//localhost:8080/job/foo/SHARD_NUM=1/1/'
    }, {
        'number': 2,
        'url': 'http//localhost:8080/job/foo/SHARD_NUM=1/2/'
    }]
}


BUILD_SCM_DATA = {
    'actions': [{
        'causes': [{'shortDescription': 'Started by an SCM change'}]
    }, {
    }, {
        'buildsByBranchName': {
            'origin/HEAD': {
                'buildNumber': 2,
                'buildResult': None,
                'revision': {
                    'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2',
                    'branch': [
                        {'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2',
                         'name': 'origin/HEAD'},
                        {'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2',
                         'name': 'origin/master'}
                    ]
                }
            },
            'origin/master': {
                'buildNumber': 2,
                'buildResult': None,
                'revision': {
                    'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2',
                    'branch': [
                        {'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2',
                         'name': 'origin/HEAD'},
                        {'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2',
                         'name': 'origin/master'}
                    ]
                }
            },
            'origin/python_3_compatibility': {
                'buildNumber': 1,
                'buildResult': None,
                'revision': {
                    'SHA1': 'c9d1c96bc926ff63a5209c51b3ed537e62ea50e6',
                    'branch': [
                        {'SHA1': 'c9d1c96bc926ff63a5209c51b3ed537e62ea50e6',
                         'name': 'origin/python_3_compatibility'}
                    ]
                }
            },
            'origin/unstable': {
                'buildNumber': 3,
                'buildResult': None,
                'revision': {
                    'SHA1': '7def9ed6e92580f37d00e4980c36c4d36e68f702',
                    'branch': [
                        {'SHA1': '7def9ed6e92580f37d00e4980c36c4d36e68f702',
                         'name': 'origin/unstable'}
                    ]
                }
            }
        },
        'lastBuiltRevision': {
            'SHA1': '7def9ed6e92580f37d00e4980c36c4d36e68f702',
            'branch': [
                {'SHA1': '7def9ed6e92580f37d00e4980c36c4d36e68f702',
                 'name': 'origin/unstable'}
            ]
        },
        'remoteUrls': [
            'https://github.com/salimfadhley/jenkinsapi.git'
        ],
        'scmName': ''
    }, {
    }, {
    }],
    'artifacts': [],
    'building': False,
    'builtOn': '',
    'changeSet': {'items': [], 'kind': 'git'},
    'culprits': [],
    'description': None,
    'duration': 1051,
    'estimatedDuration': 2260,
    'executor': None,
    'fullDisplayName': 'git_yssrtigfds #3',
    'id': '2013-06-30_01-54-35',
    'keepLog': False,
    'number': 3,
    'result': 'SUCCESS',
    'timestamp': 1372553675652,
    'url': 'http://localhost:8080/job/git_yssrtigfds/3/'
}

BUILD_ENV_VARS = {
    '_class': 'org.jenkinsci.plugins.envinject.EnvInjectVarList',
    'envMap': {'KEY': 'VALUE'}
}



import pytest
from jenkinsapi.node import Node


DATA = {
    "actions": [],
    "displayName": "bobnit",
    "executors": [{}],
    "icon": "computer.png",
    "idle": True,
    "jnlpAgent": False,
    "launchSupported": True,
    "loadStatistics": {},
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.SwapSpaceMonitor": {
            "availablePhysicalMemory": 7681417216,
            "availableSwapSpace": 12195983360,
            "totalPhysicalMemory": 8374497280,
            "totalSwapSpace": 12195983360
        },
        "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
        "hudson.node_monitors.ResponseTimeMonitor": {"average": 64},
        "hudson.node_monitors.TemporarySpaceMonitor": {
            "path": "/tmp", "size": 250172776448
        },
        "hudson.node_monitors.DiskSpaceMonitor": {
            "path": "/home/sal/jenkins",
            "size": 170472026112
        },
        "hudson.node_monitors.ClockMonitor": {"diff": 6736}
    },
    "numExecutors": 1,
    "offline": False,
    "offlineCause": None,
    "oneOffExecutors": [],
    "temporarilyOffline": False
}


@pytest.fixture(scope='function')
def node(monkeypatch, mocker):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return DATA

    monkeypatch.setattr(Node, '_poll', fake_poll)
    jenkins = mocker.MagicMock()

    return Node(jenkins, 'http://foo:8080', 'bobnit', {})


def test_repr(node):
    # Can we produce a repr string for this object
    repr(node)


def test_name(node):
    with pytest.raises(AttributeError):
        node.id()
    assert node.name == 'bobnit'


def test_online(node):
    assert node.is_online() is True


def test_available_physical_memory(node):
    monitor = DATA['monitorData']['hudson.node_monitors.SwapSpaceMonitor']
    expected_value = monitor['availablePhysicalMemory']
    assert node.get_available_physical_memory() == expected_value


def test_available_swap_space(node):
    monitor = DATA['monitorData']['hudson.node_monitors.SwapSpaceMonitor']
    expected_value = monitor['availableSwapSpace']
    assert node.get_available_swap_space() == expected_value


def test_total_physical_memory(node):
    monitor = DATA['monitorData']['hudson.node_monitors.SwapSpaceMonitor']
    expected_value = monitor['totalPhysicalMemory']
    assert node.get_total_physical_memory() == expected_value


def test_total_swap_space(node):
    monitor = DATA['monitorData']['hudson.node_monitors.SwapSpaceMonitor']
    expected_value = monitor['totalSwapSpace']
    assert node.get_total_swap_space() == expected_value


def test_workspace_path(node):
    monitor = DATA['monitorData']['hudson.node_monitors.DiskSpaceMonitor']
    expected_value = monitor['path']
    assert node.get_workspace_path() == expected_value


def test_workspace_size(node):
    monitor = DATA['monitorData']['hudson.node_monitors.DiskSpaceMonitor']
    expected_value = monitor['size']
    assert node.get_workspace_size() == expected_value


def test_temp_path(node):
    monitor = DATA['monitorData']['hudson.node_monitors.TemporarySpaceMonitor']
    expected_value = monitor['path']
    assert node.get_temp_path() == expected_value


def test_temp_size(node):
    monitor = DATA['monitorData']['hudson.node_monitors.TemporarySpaceMonitor']
    expected_value = monitor['size']
    assert node.get_temp_size() == expected_value


def test_architecture(node):
    expected_value = DATA['monitorData']['hudson.node_monitors.ArchitectureMonitor']
    assert node.get_architecture() == expected_value


def test_response_time(node):
    monitor = DATA['monitorData']['hudson.node_monitors.ResponseTimeMonitor']
    expected_value = monitor['average']
    assert node.get_response_time() == expected_value


def test_clock_difference(node):
    monitor = DATA['monitorData']['hudson.node_monitors.ClockMonitor']
    expected_value = monitor['diff']
    assert node.get_clock_difference() == expected_value



"""
jenkinsapi_tests.test_plugins
"""
import mock

# To run unittests on python 2.6 please use unittest2 library
try:
    import unittest2 as unittest
except ImportError:
    import unittest
try:
    from StringIO import StringIO  # python2
except ImportError:
    from io import BytesIO as StringIO  # python3
import zipfile

from jenkinsapi.jenkins import Requester
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.plugins import Plugins
from jenkinsapi.plugin import Plugin


class TestPlugins(unittest.TestCase):
    DATA = {
        'plugins': [
            {
                'deleted': False,
                'hasUpdate': True,
                'downgradable': False,
                'dependencies': [{}, {}, {}, {}],
                'longName': 'Jenkins Subversion Plug-in',
                'active': True,
                'shortName': 'subversion',
                'backupVersion': None,
                'url': 'http://wiki.jenkins-ci.org/display/'
                'JENKINS/Subversion+Plugin',
                'enabled': True,
                'pinned': False,
                'version': '1.45',
                'supportsDynamicLoad': 'MAYBE',
                'bundled': True
            },
            {
                'deleted': False,
                'hasUpdate': True,
                'downgradable': False,
                'dependencies': [{}, {}],
                'longName': 'Maven Integration plugin',
                'active': True,
                'shortName': 'maven-plugin',
                'backupVersion': None,
                'url': 'http://wiki.jenkins-ci.org/display/JENKINS/'
                'Maven+Project+Plugin',
                'enabled': True,
                'pinned': False,
                'version': '1.521',
                'supportsDynamicLoad': 'MAYBE',
                'bundled': True
            }
        ]
    }

    @mock.patch.object(Jenkins, '_poll')
    def setUp(self, _poll_jenkins):
        _poll_jenkins.return_value = {}

        self.J = Jenkins('http://localhost:8080')

    @mock.patch.object(Plugins, '_poll')
    def test_get_plugins(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        # Can we produce a repr string for this object
        self.assertIsInstance(self.J.get_plugins(), Plugins)

    @mock.patch.object(Plugins, '_poll')
    def test_no_plugins_str(self, _poll_plugins):
        _poll_plugins.return_value = {}

        plugins = self.J.get_plugins()
        self.assertEqual(str(plugins), "[]")

    @mock.patch.object(Plugins, '_poll')
    def test_plugins_str(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        plugins = self.J.get_plugins()
        self.assertEqual(str(plugins), "['maven-plugin', 'subversion']")

    @mock.patch.object(Plugins, '_poll')
    def test_plugins_len(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        plugins = self.J.get_plugins()
        self.assertEqual(len(plugins), 2)

    @mock.patch.object(Plugins, '_poll')
    def test_plugins_contains(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        plugins = self.J.get_plugins()
        self.assertIn('subversion', plugins)
        self.assertIn('maven-plugin', plugins)

    @mock.patch.object(Plugins, '_poll')
    def test_plugins_values(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        p = Plugin(
            {
                'deleted': False,
                'hasUpdate': True,
                'downgradable': False,
                'dependencies': [{}, {}, {}, {}],
                'longName': 'Jenkins Subversion Plug-in',
                'active': True,
                'shortName': 'subversion',
                'backupVersion': None,
                'url': 'http://wiki.jenkins-ci.org/display/JENKINS/'
                'Subversion+Plugin',
                'enabled': True,
                'pinned': False,
                'version': '1.45',
                'supportsDynamicLoad': 'MAYBE',
                'bundled': True
            }
        )

        plugins = self.J.get_plugins().values()
        self.assertIn(p, plugins)

    @mock.patch.object(Plugins, '_poll')
    def test_plugins_keys(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        plugins = self.J.get_plugins().keys()
        self.assertIn('subversion', plugins)
        self.assertIn('maven-plugin', plugins)

    @mock.patch.object(Plugins, '_poll')
    def test_plugins_empty(self, _poll_plugins):
        _poll_plugins.return_value = {}

        # list() is required here for python 3.x compatibility
        plugins = list(self.J.get_plugins().keys())
        self.assertEqual([], plugins)

    @mock.patch.object(Plugins, '_poll')
    def test_plugin_get_by_name(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA

        p = Plugin(
            {
                'deleted': False,
                'hasUpdate': True,
                'downgradable': False,
                'dependencies': [{}, {}, {}, {}],
                'longName': 'Jenkins Subversion Plug-in',
                'active': True,
                'shortName': 'subversion',
                'backupVersion': None,
                'url': 'http://wiki.jenkins-ci.org/display/JENKINS/'
                'Subversion+Plugin',
                'enabled': True,
                'pinned': False,
                'version': '1.45',
                'supportsDynamicLoad': 'MAYBE',
                'bundled': True
            }
        )

        plugin = self.J.get_plugins()['subversion']
        self.assertEqual(p, plugin)

    @mock.patch.object(Plugins, '_poll')
    def test_get_plugin_details(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA
        plugin = self.J.get_plugins()['subversion']
        self.assertEqual('1.45', plugin.version)
        self.assertEqual('subversion', plugin.shortName)
        self.assertEqual('Jenkins Subversion Plug-in', plugin.longName)
        self.assertEqual('http://wiki.jenkins-ci.org/display/JENKINS/'
                         'Subversion+Plugin',
                         plugin.url)

    @mock.patch.object(Requester, 'post_xml_and_confirm_status')
    def test_install_plugin_bad_input(self, _post):
        with self.assertRaises(ValueError):
            self.J.install_plugin('test')

    @mock.patch.object(Requester, 'post_xml_and_confirm_status')
    def test_delete_plugin_bad_input(self, _post):
        with self.assertRaises(ValueError):
            self.J.delete_plugin('test@latest')

    @mock.patch.object(Plugins, '_poll')
    @mock.patch.object(Plugins, 'plugin_version_already_installed')
    @mock.patch.object(Plugins, 'restart_required')
    @mock.patch.object(Plugins, '_wait_until_plugin_installed')
    @mock.patch.object(Requester, 'post_xml_and_confirm_status')
    @mock.patch.object(Jenkins, 'safe_restart')
    def test_install_plugin_good_input(self, _reboot, _post, _wait,
                                       _restart_required, already_installed,
                                       _poll_plugins):
        _poll_plugins.return_value = self.DATA
        already_installed.return_value = False
        self.J.install_plugin('test@latest')
        expected_data = '<jenkins> <install plugin="test@latest" /> </jenkins>'
        _post.assert_called_with(
            '/'.join([self.J.baseurl,
                      'pluginManager',
                      'installNecessaryPlugins']),
            data=expected_data)

    @mock.patch.object(Plugins, '_poll')
    @mock.patch.object(Plugins, 'plugin_version_already_installed')
    @mock.patch.object(Plugins, 'restart_required',
                       new_callable=mock.mock.PropertyMock)
    @mock.patch.object(Plugins, '_wait_until_plugin_installed')
    @mock.patch.object(Requester, 'post_xml_and_confirm_status')
    @mock.patch.object(Jenkins, 'safe_restart')
    def test_install_plugins_good_input_no_restart_required(self, _restart,
                                                            _post, _wait,
                                                            restart_required,
                                                            already_installed,
                                                            _poll_plugins):
        _poll_plugins.return_value = self.DATA
        restart_required.return_value = False
        already_installed.return_value = False
        self.J.install_plugins(['test@latest', 'test@latest'])
        self.assertEqual(_post.call_count, 2)
        self.assertEqual(_restart.call_count, 0)

    @mock.patch.object(Plugins, '_poll')
    @mock.patch.object(Plugins, 'plugin_version_already_installed')
    @mock.patch.object(Plugins, 'restart_required',
                       new_callable=mock.mock.PropertyMock)
    @mock.patch.object(Plugins, '_wait_until_plugin_installed')
    @mock.patch.object(Requester, 'post_xml_and_confirm_status')
    @mock.patch.object(Jenkins, 'safe_restart')
    def test_install_plugins_good_input_with_restart_required(self, _restart,
                                                              _post, _wait,
                                                              restart_required,
                                                              already_installed,
                                                              _poll_plugins):
        _poll_plugins.return_value = self.DATA
        restart_required.return_value = True
        already_installed.return_value = False
        self.J.install_plugins(['test@latest', 'test@latest'])
        self.assertEqual(_post.call_count, 2)
        self.assertEqual(_restart.call_count, 1)

    @mock.patch.object(Plugins, '_poll')
    def test_get_plugin_dependencies(self, _poll_plugins):
        manifest = (
            'Manifest-Version: 1.0\n'
            'bla: somestuff\n'
            'Plugin-Dependencies: aws-java-sdk:1.10.45,aws-credentials:1.15'
        )
        downloaded_plugin = StringIO()
        zipfile.ZipFile(
            downloaded_plugin, mode='w').writestr(
                'META-INF/MANIFEST.MF', manifest)
        _poll_plugins.return_value = self.DATA
        dependencies = self.J.plugins._get_plugin_dependencies(
            downloaded_plugin)
        self.assertEqual(len(dependencies), 2)
        for dep in dependencies:
            self.assertIsInstance(dep, Plugin)

    @mock.patch.object(Plugins, '_poll')
    def test_plugin_version_already_installed(self, _poll_plugins):
        _poll_plugins.return_value = self.DATA
        already_installed = Plugin({'shortName': 'subversion',
                                    'version': '1.45'})
        self.assertTrue(
            self.J.plugins.plugin_version_already_installed(already_installed))
        not_installed = Plugin({'shortName': 'subversion', 'version': '1.46'})
        self.assertFalse(
            self.J.plugins.plugin_version_already_installed(not_installed))
        latest = Plugin({'shortName': 'subversion', 'version': 'latest'})
        self.assertFalse(
            self.J.plugins.plugin_version_already_installed(latest))

    @mock.patch.object(Plugins, '_poll')
    @mock.patch.object(Plugins, 'update_center_install_status',
                       new_callable=mock.mock.PropertyMock)
    def test_restart_required_after_plugin_installation(self, status,
                                                        _poll_plugins):
        _poll_plugins.return_value = self.DATA
        status.return_value = {
            'data': {
                'jobs': [{
                    'installStatus': 'SuccessButRequiresRestart',
                    'name': 'credentials',
                    'requiresRestart': 'true',
                    'title': None,
                    'version': '0'
                }],
                'state': 'RUNNING'
            },
            'status': 'ok'
        }
        self.assertTrue(self.J.plugins.restart_required)

    @mock.patch.object(Plugins, '_poll')
    @mock.patch.object(Plugins, 'update_center_install_status',
                       new_callable=mock.mock.PropertyMock)
    def test_restart_not_required_after_plugin_installation(self, status,
                                                            _poll_plugins):
        _poll_plugins.return_value = self.DATA
        status.return_value = {'data': {'jobs': [],
                                        'state': 'RUNNING'},
                               'status': 'ok'}
        self.assertFalse(self.J.plugins.restart_required)

    def test_plugin_repr(self):
        p = Plugin(
            {
                'shortName': 'subversion',
            }
        )
        self.assertEqual(repr(p), '<jenkinsapi.plugin.Plugin subversion>')


if __name__ == '__main__':
    unittest.main()



"""
Start a Parameterized Build
"""
from __future__ import print_function

from jenkinsapi.jenkins import Jenkins

jenkins = Jenkins('http://localhost:8080')

params = {'VERSION': '1.2.3', 'PYTHON_VER': '2.7'}

# This will start the job in non-blocking manner
jenkins.build_job('foo', params)


# This will start the job and will return a QueueItem object which
# can be used to get build results
job = jenkins['foo']
qi = job.invoke(build_params=params)

# Block this script until build is finished
if qi.is_queued() or qi.is_running():
    qi.block_until_complete()

build = qi.get_build()
print(build)



import jenkinsapi


def test_jenkinsapi_version():
    """Verify that we can get the jenkinsapi version number from the
    package's __version__ property.
    """
    version = jenkinsapi.__version__
    # only first two parts must be interger, 1.0.dev5 being a valid version.
    parts = [int(x) for x in version.split('.')[0:2]]
    for part in parts:
        assert part >= 0, "Implausible version number: %r" % version



"""
System tests for `jenkinsapi.jenkins` module.
"""
import time
import logging
import pytest
from jenkinsapi.build import Build
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi_tests.systests.job_configs import LONG_RUNNING_JOB


log = logging.getLogger(__name__)


@pytest.mark.run_these_please
def test_safe_exit(jenkins):
    job_name = 'Bcreate_%s' % random_string()
    job = jenkins.create_job(job_name, LONG_RUNNING_JOB)
    qq = job.invoke()
    time.sleep(3)
    bn = qq.block_until_building(delay=3).get_number()
    assert isinstance(bn, int)

    build = qq.get_build()
    assert isinstance(build, Build)
    assert build.is_running()

    # A job is now running and safe_exit should await running jobs
    # Call, but wait only for 5 seconds then cancel exit
    jenkins.safe_exit(wait_for_exit=False)
    time.sleep(5)

    jenkins.cancel_quiet_down()  # leave quietDown mode
    assert jenkins.is_quieting_down is False

    build.stop()
    # if we call next line right away - Jenkins have no time to stop job
    # so we wait a bit
    while build.is_running():
        time.sleep(0.5)

    console = build.get_console()
    assert isinstance(console, str)
    assert 'Started by user' in console



'''
System tests for `jenkinsapi.plugins` module.
'''
import logging
import pytest
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi.plugin import Plugin

log = logging.getLogger(__name__)


def test_plugin_data(jenkins):
    # It takes time to get plugins json from remote
    timeout = jenkins.requester.timeout
    jenkins.requester.timeout = 60
    jenkins.plugins.check_updates_server()
    jenkins.requester.timeout = timeout

    assert 'mailer' in jenkins.plugins


def test_get_missing_plugin(jenkins):
    plugins = jenkins.get_plugins()
    with pytest.raises(KeyError):
        plugins["lsdajdaslkjdlkasj"]  # this plugin surely does not exist!


def test_get_single_plugin(jenkins):
    plugins = jenkins.get_plugins()
    plugin_name, plugin = next(plugins.iteritems())

    assert isinstance(plugin_name, str)
    assert isinstance(plugin, Plugin)


def test_get_single_plugin_depth_2(jenkins):
    plugins = jenkins.get_plugins(depth=2)
    _, plugin = next(plugins.iteritems())

    assert isinstance(plugin, Plugin)


def test_delete_inexistant_plugin(jenkins):
    with pytest.raises(KeyError):
        del jenkins.plugins[random_string()]


def test_install_uninstall_plugin(jenkins):
    plugin_name = 'suppress-stack-trace'

    plugin_dict = {
        'shortName': plugin_name,
        'version': 'latest',
    }
    jenkins.plugins[plugin_name] = Plugin(plugin_dict)

    assert plugin_name in jenkins.plugins

    plugin = jenkins.get_plugins()[plugin_name]
    assert isinstance(plugin, Plugin)
    assert plugin.shortName == plugin_name

    del jenkins.plugins[plugin_name]
    assert jenkins.plugins[plugin_name].deleted


def test_install_multiple_plugins(jenkins):
    plugin_one_name = 'keyboard-shortcuts-plugin'
    plugin_one_version = 'latest'
    plugin_one = "@".join((plugin_one_name, plugin_one_version))
    plugin_two = Plugin({'shortName': 'emotional-jenkins-plugin', 'version': 'latest'})

    assert isinstance(plugin_two, Plugin)

    plugin_list = [plugin_one, plugin_two]

    jenkins.install_plugins(plugin_list)

    assert plugin_one_name in jenkins.plugins
    assert plugin_two.shortName in jenkins.plugins

    del jenkins.plugins['keyboard-shortcuts-plugin']
    del jenkins.plugins['emotional-jenkins-plugin']


def test_downgrade_plugin(jenkins):
    plugin_name = 'console-badge'
    plugin_version = 'latest'
    plugin = Plugin({'shortName': plugin_name, 'version': plugin_version})

    assert isinstance(plugin, Plugin)

    # Need to restart when not installing the latest version
    jenkins.install_plugins([plugin])

    installed_plugin = jenkins.plugins[plugin_name]

    assert installed_plugin.version == '1.1'

    older_plugin = Plugin({'shortName': plugin_name, 'version': '1.0'})
    jenkins.install_plugins([older_plugin], restart=True, wait_for_reboot=True)
    installed_older_plugin = jenkins.plugins[plugin_name]

    assert installed_older_plugin.version == '1.0'

    del jenkins.plugins[plugin_name]



"""
System tests for `jenkinsapi.jenkins` module.
"""
import logging
import pytest
from jenkinsapi.node import Node
from jenkinsapi.credential import SSHKeyCredential
from jenkinsapi_tests.test_utils.random_strings import random_string

log = logging.getLogger(__name__)


def test_online_offline(jenkins):
    """
    Can we flip the online / offline state of the master node.
    """
    # Master node name should be case insensitive
    # mn0 = jenkins.get_node('MaStEr')
    mn = jenkins.get_node('master')
    # self.assertEqual(mn, mn0)

    mn.set_online()  # It should already be online, hence no-op
    assert mn.is_online() is True

    mn.set_offline()  # We switch that suckah off
    mn.set_offline()  # This should be a no-op
    assert mn.is_online() is False

    mn.set_online()  # Switch it back on
    assert mn.is_online() is True


def test_create_jnlp_node(jenkins):
    node_name = random_string()
    node_dict = {
        'num_executors': 1,
        'node_description': 'Test JNLP Node',
        'remote_fs': '/tmp',
        'labels': 'systest_jnlp',
        'exclusive': True,
        'tool_location': [
            {
                "key": "hudson.tasks.Maven$MavenInstallation$DescriptorImpl@Maven 3.0.5",
                "home": "/home/apache-maven-3.0.5/"
            },
        ]
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    assert isinstance(node, Node) is True

    del jenkins.nodes[node_name]


def test_create_ssh_node(jenkins):
    node_name = random_string()
    creds = jenkins.get_credentials()

    cred_descr = random_string()
    cred_dict = {
        'description': cred_descr,
        'userName': 'username',
        'passphrase': '',
        'private_key': '-----BEGIN RSA PRIVATE KEY-----'
    }
    creds[cred_descr] = SSHKeyCredential(cred_dict)
    node_dict = {
        'num_executors': 1,
        'node_description': 'Description %s' % node_name,
        'remote_fs': '/tmp',
        'labels': node_name,
        'exclusive': False,
        'host': '127.0.0.1',
        'port': 22,
        'credential_description': cred_descr,
        'jvm_options': '',
        'java_path': '',
        'prefix_start_slave_cmd': '',
        'suffix_start_slave_cmd': '',
        'retention': 'ondemand',
        'ondemand_delay': 0,
        'ondemand_idle_delay': 5,
        'tool_location': [
            {
                "key": "hudson.tasks.Maven$MavenInstallation$DescriptorImpl@Maven 3.0.5",
                "home": "/home/apache-maven-3.0.5/"
            },
        ]
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    assert isinstance(node, Node) is True
    del jenkins.nodes[node_name]

    jenkins.nodes[node_name] = node_dict
    assert isinstance(jenkins.nodes[node_name], Node) is True
    del jenkins.nodes[node_name]


def test_delete_node(jenkins):
    node_name = random_string()
    node_dict = {
        'num_executors': 1,
        'node_description': 'Test JNLP Node',
        'remote_fs': '/tmp',
        'labels': 'systest_jnlp',
        'exclusive': True
    }
    jenkins.nodes.create_node(node_name, node_dict)
    del jenkins.nodes[node_name]

    with pytest.raises(KeyError):
        jenkins.nodes[node_name]

    with pytest.raises(KeyError):
        del jenkins.nodes['not_exist']


def test_delete_all_nodes(jenkins):
    nodes = jenkins.nodes

    for name in nodes.keys():
        del nodes[name]

    assert len(jenkins.nodes) == 1


def test_get_node_labels(jenkins):
    node_name = random_string()
    node_labels = 'LABEL1 LABEL2'
    node_dict = {
        'num_executors': 1,
        'node_description': 'Test Node with Labels',
        'remote_fs': '/tmp',
        'labels': node_labels,
        'exclusive': True
    }
    node = jenkins.nodes.create_node(node_name, node_dict)
    assert node.get_labels() == node_labels

    del jenkins.nodes[node_name]


def test_get_executors(jenkins):
    node_name = random_string()
    node_labels = 'LABEL1 LABEL2'
    node_dict = {
        'num_executors': 1,
        'node_description': 'Test Node with Labels',
        'remote_fs': '/tmp',
        'labels': node_labels,
        'exclusive': True
    }
    node = jenkins.nodes.create_node(node_name, node_dict)

    with pytest.raises(AttributeError):
        assert node.get_config_element('executors') == '1'

    assert node.get_config_element('numExecutors') == '1'

    del jenkins.nodes[node_name]


def test_set_executors(jenkins):
    node_name = random_string()
    node_labels = 'LABEL1 LABEL2'
    node_dict = {
        'num_executors': 1,
        'node_description': 'Test Node with Labels',
        'remote_fs': '/tmp',
        'labels': node_labels,
        'exclusive': True
    }
    node = jenkins.nodes.create_node(node_name, node_dict)

    assert node.set_config_element('numExecutors', '5') is None

    assert node.get_config_element('numExecutors') == '5'

    del jenkins.nodes[node_name]


def test_set_master_executors(jenkins):
    node = jenkins.nodes['master']

    assert node.get_num_executors() == 2

    node.set_num_executors(5)
    assert node.get_num_executors() == 5

    node.set_num_executors(2)


def test_offline_reason(jenkins):
    node_name = random_string()
    node_labels = 'LABEL1 LABEL2'
    node_dict = {
        'num_executors': 1,
        'node_description': 'Test Node with Labels',
        'remote_fs': '/tmp',
        'labels': node_labels,
        'exclusive': True
    }
    node = jenkins.nodes.create_node(node_name, node_dict)

    node.toggle_temporarily_offline('test1')
    node.poll()
    assert node.offline_reason() == 'test1'

    node.update_offline_reason('test2')
    node.poll()
    assert node.offline_reason() == 'test2'

    del jenkins.nodes[node_name]



import mock

import pytest

from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.view import View
from jenkinsapi.job import Job
from jenkinsapi.custom_exceptions import NotFound

DATA = {'description': 'Important Shizz',
        'jobs': [
            {'color': 'blue',
             'name': 'foo',
             'url': 'http://halob:8080/job/foo/'},
            {'color': 'red',
             'name': 'test_jenkinsapi',
             'url': 'http://halob:8080/job/test_jenkinsapi/'}
        ],
        'name': 'FodFanFo',
        'property': [],
        'url': 'http://halob:8080/view/FodFanFo/'}

JOB_DATA = {
    "actions": [],
    "description": "test job",
    "displayName": "foo",
    "displayNameOrNull": None,
    "name": "foo",
    "url": "http://halob:8080/job/foo/",
    "buildable": True,
    "builds": [
        {"number": 3, "url": "http://halob:8080/job/foo/3/"},
        {"number": 2, "url": "http://halob:8080/job/foo/2/"},
        {"number": 1, "url": "http://halob:8080/job/foo/1/"}
    ],
    "color": "blue",
    "firstBuild": {"number": 1, "url": "http://halob:8080/job/foo/1/"},
    "healthReport": [
        {"description": "Build stability: No recent builds failed.",
         "iconUrl": "health-80plus.png", "score": 100}
    ],
    "inQueue": False,
    "keepDependencies": False,
    "lastBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
    "lastCompletedBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
    "lastFailedBuild": None,
    "lastStableBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
    "lastSuccessfulBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
    "lastUnstableBuild": None,
    "lastUnsuccessfulBuild": None,
    "nextBuildNumber": 4,
    "property": [],
    "queueItem": None,
    "concurrentBuild": False,
    "downstreamProjects": [],
    "scm": {},
    "upstreamProjects": []
}


@pytest.fixture
def jenkins():
    jenkins = mock.MagicMock(auto_spec=True)
    jenkins.has_job.return_value = False
    return jenkins


@mock.patch.object(Job, '_poll', auto_spec=True)
@mock.patch.object(View, '_poll', auto_spec=True)
@pytest.fixture
def view(_view_poll, _job_poll, jenkins):
    _view_poll.return_value = DATA
    _job_poll.return_value = JOB_DATA
    return View('http://localhost:800/view/FodFanFo',
                'FodFanFo',
                jenkins)


@pytest.fixture
def jenkins_patch():

    class Jenkins:
        def has_job(self, job_name):
            return False

        def get_jenkins_obj_from_url(self, url):
            return self

    return Jenkins


@pytest.fixture
def busy_patch():

    class Jenkins:
        def has_job(self, job_name):
            return True

        def get_jenkins_obj_from_url(self, url):
            return self

    return Jenkins


class TestView:

    def test_returns_name_when_repr_is_called(self, view):
        assert repr(view) == 'FodFanFo'

    def test_returns_name_when_str_method_called(self, view):
        assert str(view) == 'FodFanFo'

    def test_raises_error_when_is_called(self, view):
        with pytest.raises(AttributeError):
            view.id()

    def test_returns_name_when_name_property_is_called(self, view):
        assert view.name == 'FodFanFo'

    @mock.patch.object(JenkinsBase, '_poll')
    def test_iteritems(self, _poll, view):
        _poll.return_value = JOB_DATA
        for job_name, job_obj in view.iteritems():
            assert isinstance(job_obj, Job)
            assert job_name in ['foo', 'test_jenkinsapi']

    def test_returns_dict_of_job_info_when_job_dict_method_called(self, view):
        jobs = view.get_job_dict()

        assert jobs == {
            'foo': 'http://halob:8080/job/foo/',
            'test_jenkinsapi': 'http://halob:8080/job/test_jenkinsapi/'
        }

    def test_returns_len_when_len_is_called(self, view):
        assert len(view) == 2

    @mock.patch.object(JenkinsBase, '_poll')
    def test_getitem(self, _poll, view):
        _poll.return_value = JOB_DATA

        assert isinstance(view['foo'], Job)

    def test_sets_delete_to_true_when_deleted(self, view):
        view.delete()

        assert view.deleted

    def test_returns_url_when_get_job_url_is_called(self, view):
        url = view.get_job_url('foo')

        assert url == 'http://halob:8080/job/foo/'

    def test_raises_not_found_when_get_job_url_is_invalid(self, view):
        with pytest.raises(NotFound):
            view.get_job_url('bar')

    @mock.patch.object(View, 'get_jenkins_obj')
    def test_returns_false_when_adding_wrong_job(
            self, _get_jenkins, view, jenkins_patch):
        _get_jenkins.return_value = jenkins_patch()
        result = view.add_job('bar')

        assert result is False

    def test_returns_false_when_add_existing_job(self, view):
        result = view.add_job('foo')

        assert result is False

    def test_get_nested_view_dict(self, view):
        result = view.get_nested_view_dict()

        assert isinstance(result, dict)

    def test_returns_jenkins_obj_when_get_jenkins_obj_is_called(self, view):
        obj = view.get_jenkins_obj()

        assert obj == view.jenkins_obj


class TestKeys:

    def test_returns_key_when_called(self, view):
        keys = view.keys()

        assert 'foo' in list(keys)
        assert 'test_jenkinsapi' in list(keys)


class TestAddJob:

    @mock.patch.object(JenkinsBase, '_poll')
    def test_returns_true_when_no_job_provided(self, _poll, view):
        _poll.return_value = DATA

        result = view.add_job('bar')

        assert result is True

    @mock.patch.object(JenkinsBase, '_poll')
    def test_returns_false_when_already_registered(self, _poll, view):
        _poll.return_value = DATA

        result = view.add_job('foo')

        assert result is False

    @mock.patch.object(View, 'get_jenkins_obj')
    def test_returns_false_when_jenkins_has_job(self, _get_jenkins, view, jenkins_patch):
        _get_jenkins.return_value = jenkins_patch()

        result = view.add_job('Foo')

        _get_jenkins.assert_called()
        assert result is False

    @mock.patch.object(View, 'get_jenkins_obj')
    @mock.patch.object(JenkinsBase, '_poll')
    def test_returns_true_when_jenkins_has_job(self, _poll, _get_jenkins, view, jenkins):
        _get_jenkins.return_value = jenkins()
        _poll.return_value = DATA
        result = view.add_job('Foo')

        _get_jenkins.assert_called()
        assert result is True


class TestRemove:
    @mock.patch.object(JenkinsBase, '_poll')
    def test_returns_true_when_job_has_been_removed(self, _poll, view):
        _poll.return_value = DATA

        result = view.remove_job('foo')

        assert result is True

    @mock.patch.object(JenkinsBase, '_poll')
    def test_returns_false_when_job_does_not_exist(self, _poll, view):
        _poll.return_value = DATA

        result = view.remove_job('Non-existant Foo')

        assert result is False



import pytest
from jenkinsapi.label import Label


DATA = {
    "actions": [],
    "busyExecutors": 0,
    "clouds": [],
    "description": None,
    "idleExecutors": 0,
    "loadStatistics": {},
    "name": "jenkins-slave",
    "nodes": [],
    "offline": True,
    "tiedJobs": [
        {
            "name": "test_job1",
            "url": "http://jtest:8080/job/test_job1/",
            "color": "blue"
        }, {
            "name": "test_job2",
            "url": "http://jtest:8080/job/test_job2/",
            "color": "blue"
        }, {
            "name": "test_job3",
            "url": "http://jtest:8080/job/test_job3/",
            "color": "blue"
        }, {
            "name": "test_job4",
            "url": "http://jtest:8080/job/test_job4/",
            "color": "blue"
        }
    ],
    "totalExecutors": 0,
    "propertiesList": []
}

DATA_JOB_NAMES = {
    "tiedJobs": [
        {"name": "test_job1"},
        {"name": "test_job2"},
        {"name": "test_job3"},
        {"name": "test_job4"}
    ]
}

DATA_JOBS = [
    {
        "url": "http://jtest:8080/job/test_job1/",
        "color": "blue",
        "name": "test_job1"
    }, {
        "url": "http://jtest:8080/job/test_job2/",
        "color": "blue",
        "name": "test_job2"
    }, {
        "url": "http://jtest:8080/job/test_job3/",
        "color": "blue",
        "name": "test_job3"
    }, {
        "url": "http://jtest:8080/job/test_job4/",
        "color": "blue",
        "name": "test_job4"
    }
]


@pytest.fixture(scope='function')
def label(monkeypatch, mocker):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return DATA

    monkeypatch.setattr(Label, '_poll', fake_poll)
    jenkins = mocker.MagicMock()

    return Label('http://foo:8080', 'jenkins-slave', jenkins)


def test_repr(label):
    # Can we produce a repr string for this object
    repr(label)


def test_name(label):
    with pytest.raises(AttributeError):
        label.id()
    assert label.labelname == 'jenkins-slave'


def test_get_tied_job_names(label):
    assert label.get_tied_job_names() == DATA_JOBS


def test_online(label):
    assert label.is_online() is False



"""
Example of using CrumbRequester - when CSRF protection is enabled in Jenkins
"""
from jenkinsapi.jenkins import Jenkins

jenkins = Jenkins('http://localhost:8080', username='admin', password='password', useCrumb=True)

for job_name in jenkins.jobs:
    print(job_name)



"""
Module for Python 2 and Python 3 compatibility
"""
import six


def needs_encoding(data):
    """
    Check whether data is Python 2 unicode variable and needs to be
    encoded
    """
    if six.PY2 and isinstance(data, unicode):
        return True
    return False


def to_string(data, encoding='utf-8'):
    """
    Return string representation for the data. In case of Python 2 and unicode
    do additional encoding before
    """
    encoded_text = data.encode(encoding) if needs_encoding(data) else data
    return str(encoded_text)



import pytest
from mock import (
    Mock,
    patch,
    call
)
from requests.exceptions import HTTPError
from jenkinsapi.artifact import Artifact
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.fingerprint import Fingerprint
from jenkinsapi.custom_exceptions import ArtifactBroken
try:
    import unittest2 as unittest
except ImportError:
    import unittest


@pytest.fixture()
def artifact(mocker):
    return Artifact(
        'artifact.zip', 'http://foo/job/TestJob/1/artifact/artifact.zip',
        mocker.MagicMock()
    )


def test_verify_download_valid_positive(artifact, monkeypatch):
    def fake_md5(cls, fspath):   # pylint: disable=unused-argument
        return '097c42989a9e5d9dcced7b35ec4b0486'

    monkeypatch.setattr(Artifact, '_md5sum', fake_md5)

    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, '_poll', fake_poll)

    def fake_validate(cls, filename,   # pylint: disable=unused-argument
                      job, build):   # pylint: disable=unused-argument
        return True

    monkeypatch.setattr(Fingerprint, 'validate_for_build', fake_validate)

    assert artifact._verify_download('/tmp/artifact.zip',
                                     strict_validation=False)


def test_verify_download_valid_positive_with_rename(artifact, monkeypatch):
    def fake_md5(cls, fspath):   # pylint: disable=unused-argument
        return '097c42989a9e5d9dcced7b35ec4b0486'

    monkeypatch.setattr(Artifact, '_md5sum', fake_md5)

    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, '_poll', fake_poll)

    def fake_validate(cls, filename,   # pylint: disable=unused-argument
                      job, build):   # pylint: disable=unused-argument
        return filename == 'artifact.zip'

    monkeypatch.setattr(Fingerprint, 'validate_for_build', fake_validate)

    assert artifact._verify_download('/tmp/temporary_filename',
                                     strict_validation=False)


def test_verify_download_valid_negative(artifact, monkeypatch):
    def fake_md5(cls, fspath):   # pylint: disable=unused-argument
        return '097c42989a9e5d9dcced7b35ec4b0486'

    monkeypatch.setattr(Artifact, '_md5sum', fake_md5)

    class FakeResponse(object):
        status_code = 404
        text = '{}'

    class FakeHTTPError(HTTPError):
        def __init__(self):
            self.response = FakeResponse()

    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        raise FakeHTTPError()

    monkeypatch.setattr(JenkinsBase, '_poll', fake_poll)

    def fake_validate(cls, filename,   # pylint: disable=unused-argument
                      job, build):   # pylint: disable=unused-argument
        return True

    monkeypatch.setattr(Fingerprint, 'validate_for_build', fake_validate)

    assert artifact._verify_download('/tmp/artifact.zip',
                                     strict_validation=False)


def test_verify_dl_valid_negative_strict(artifact, monkeypatch):
    def fake_md5(cls, fspath):   # pylint: disable=unused-argument
        return '097c42989a9e5d9dcced7b35ec4b0486'

    monkeypatch.setattr(Artifact, '_md5sum', fake_md5)

    class FakeResponse(object):
        status_code = 404
        text = '{}'

    class FakeHTTPError(HTTPError):
        def __init__(self):
            self.response = FakeResponse()

    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        raise FakeHTTPError()

    monkeypatch.setattr(JenkinsBase, '_poll', fake_poll)

    with pytest.raises(ArtifactBroken) as ab:
        artifact._verify_download('/tmp/artifact.zip',
                                  strict_validation=True)

    assert 'Artifact 097c42989a9e5d9dcced7b35ec4b0486 seems to be broken' \
           in str(ab.value)


def test_verify_download_invalid(artifact, monkeypatch):
    def fake_md5(cls, fspath):   # pylint: disable=unused-argument
        return '097c42989a9e5d9dcced7b35ec4b0486'

    monkeypatch.setattr(Artifact, '_md5sum', fake_md5)

    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, '_poll', fake_poll)

    def fake_validate(cls, filename,   # pylint: disable=unused-argument
                      job, build):   # pylint: disable=unused-argument
        return False

    monkeypatch.setattr(Fingerprint, 'validate_for_build', fake_validate)

    with pytest.raises(ArtifactBroken) as ab:
        artifact._verify_download('/tmp/artifact.zip',
                                  strict_validation=True)

    assert 'Artifact 097c42989a9e5d9dcced7b35ec4b0486 seems to be broken' \
           in str(ab.value)


class ArtifactTest(unittest.TestCase):

    def setUp(self):
        self._build = build = Mock()
        build.buildno = 9999
        job = self._build.job
        job.jenkins.baseurl = 'http://localhost'
        job.name = 'TestJob'
        self._artifact = Artifact(
            'artifact.zip',
            'http://localhost/job/TestJob/9999/artifact/artifact.zip', build)

    @patch('jenkinsapi.artifact.os.path.exists', spec=True, return_value=True)
    def test_save_has_valid_local_copy(self, mock_exists):
        artifact = self._artifact
        artifact._verify_download = Mock(return_value=True)

        assert artifact.save('/tmp/artifact.zip') == '/tmp/artifact.zip'

        mock_exists.assert_called_once_with('/tmp/artifact.zip')
        artifact._verify_download.assert_called_once_with(
            '/tmp/artifact.zip', False)

    @patch('jenkinsapi.artifact.os.path.exists', spec=True, return_value=True)
    def test_save_has_invalid_local_copy_dl_again(self, mock_exists):
        artifact = self._artifact
        artifact._verify_download = Mock(side_effect=[ArtifactBroken, True])
        artifact._do_download = Mock(return_value='/tmp/artifact.zip')

        assert artifact.save('/tmp/artifact.zip', True) == '/tmp/artifact.zip'

        mock_exists.assert_called_once_with('/tmp/artifact.zip')
        artifact._do_download.assert_called_once_with('/tmp/artifact.zip')
        assert artifact._verify_download.mock_calls == \
            [call('/tmp/artifact.zip', True)] * 2

    @patch('jenkinsapi.artifact.os.path.exists', spec=True, return_value=True)
    def test_has_invalid_lcl_copy_dl_but_invalid(
            self, mock_exists):
        artifact = self._artifact
        artifact._verify_download = Mock(
            side_effect=[ArtifactBroken, ArtifactBroken])
        artifact._do_download = Mock(return_value='/tmp/artifact.zip')

        with pytest.raises(ArtifactBroken):
            artifact.save('/tmp/artifact.zip', True)

        mock_exists.assert_called_once_with('/tmp/artifact.zip')
        artifact._do_download.assert_called_once_with('/tmp/artifact.zip')
        assert artifact._verify_download.mock_calls == \
            [call('/tmp/artifact.zip', True)] * 2

    @patch('jenkinsapi.artifact.os.path.exists', spec=True, return_value=False)
    def test_save_has_no_local_copy(self, mock_exists):
        artifact = self._artifact
        artifact._do_download = Mock(return_value='/tmp/artifact.zip')
        artifact._verify_download = Mock(return_value=True)

        assert artifact.save('/tmp/artifact.zip') == '/tmp/artifact.zip'

        mock_exists.assert_called_once_with('/tmp/artifact.zip')
        artifact._do_download.assert_called_once_with('/tmp/artifact.zip')
        artifact._verify_download.assert_called_once_with(
            '/tmp/artifact.zip', False)



from __future__ import print_function

import random
import string


def random_string(length=10):
    return ''.join(random.choice(string.ascii_lowercase)
                   for i in range(length))


if __name__ == '__main__':
    print(random_string())



"""
Module for jenkinsapi Node class
"""
import json
import logging

import xml.etree.ElementTree as ET

import time
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.custom_exceptions import PostRequired, TimeOut
from jenkinsapi.custom_exceptions import JenkinsAPIException
from six.moves.urllib.parse import quote as urlquote

log = logging.getLogger(__name__)


class Node(JenkinsBase):

    """
    Class to hold information on nodes that are attached as slaves
    to the master jenkins instance
    """

    def __init__(self, jenkins_obj, baseurl, nodename, node_dict, poll=True):
        """
        Init a node object by providing all relevant pointers to it
        :param jenkins_obj: ref to the jenkins obj
        :param baseurl: basic url for querying information on a node
            If url is not set - object will construct it itself. This is
            useful when node is being created and not exists in Jenkins yet
        :param nodename: hostname of the node
        :param dict node_dict: Dict with node parameters as described below
        :param bool poll: set to False if node does not exist or automatic
            refresh from Jenkins is not required. Default is True.
            If baseurl parameter is set to None - poll parameter will be
            set to False

        JNLP Node:
            {
                'num_executors': int,
                'node_description': str,
                'remote_fs': str,
                'labels': str,
                'exclusive': bool
            }

        SSH Node:
        {
            'num_executors': int,
            'node_description': str,
            'remote_fs': str,
            'labels': str,
            'exclusive': bool,
            'host': str,
            'port': int
            'credential_description': str,
            'jvm_options': str,
            'java_path': str,
            'prefix_start_slave_cmd': str,
            'suffix_start_slave_cmd': str
            'max_num_retries': int,
            'retry_wait_time': int,
            'retention': str ('Always' or 'OnDemand')
            'ondemand_delay': int (only for OnDemand retention)
            'ondemand_idle_delay': int (only for OnDemand retention)
            'env': [
                {
                    'key':'TEST',
                    'value':'VALUE'
                },
                {
                    'key':'TEST2',
                    'value':'value2'
                }
            ],
            'tool_location': [
                {
                    "key": "hudson.tasks.Maven$MavenInstallation$DescriptorImpl@Maven 3.0.5",
                    "home": "/home/apache-maven-3.0.5/"
                },
                {
                    "key": "hudson.plugins.git.GitTool$DescriptorImpl@Default",
                    "home": "/home/git-3.0.5/"
                },
            ]
        }

        :return: None
        :return: Node obj
        """
        self.name = nodename
        self.jenkins = jenkins_obj
        if not baseurl:
            poll = False
            baseurl = '%s/computer/%s' % (self.jenkins.baseurl, self.name)
        JenkinsBase.__init__(self, baseurl, poll=poll)
        self.node_attributes = node_dict
        self._element_tree = None
        self._config = None

    def get_node_attributes(self):
        """
        Gets node attributes as dict

        Used by Nodes object when node is created

        :return: Node attributes dict formatted for Jenkins API request
            to create node
        """
        na = self.node_attributes
        if not na.get('credential_description', False):
            # If credentials description is not present - we will create
            # JNLP node
            launcher = {'stapler-class': 'hudson.slaves.JNLPLauncher'}
        else:
            try:
                credential = self.jenkins.credentials[
                    na['credential_description']
                ]
            except KeyError:
                raise JenkinsAPIException('Credential with description "%s"'
                                          ' not found'
                                          % na['credential_description'])

            retries = na['max_num_retries'] if 'max_num_retries' in na else ''
            re_wait = na['retry_wait_time'] if 'retry_wait_time' in na else ''
            launcher = {
                'stapler-class': 'hudson.plugins.sshslaves.SSHLauncher',
                '$class': 'hudson.plugins.sshslaves.SSHLauncher',
                'host': na['host'],
                'port': na['port'],
                'credentialsId': credential.credential_id,
                'jvmOptions': na['jvm_options'],
                'javaPath': na['java_path'],
                'prefixStartSlaveCmd': na['prefix_start_slave_cmd'],
                'suffixStartSlaveCmd': na['suffix_start_slave_cmd'],
                'maxNumRetries': retries,
                'retryWaitTime': re_wait
            }

        retention = {
            'stapler-class': 'hudson.slaves.RetentionStrategy$Always',
            '$class': 'hudson.slaves.RetentionStrategy$Always'
        }
        if 'retention' in na and na['retention'].lower() == 'ondemand':
            retention = {
                'stapler-class': 'hudson.slaves.RetentionStrategy$Demand',
                '$class': 'hudson.slaves.RetentionStrategy$Demand',
                'inDemandDelay': na['ondemand_delay'],
                'idleDelay': na['ondemand_idle_delay']
            }

        node_props = {
            'stapler-class-bag': 'true'
        }
        if 'env' in na:
            node_props.update({
                'hudson-slaves-EnvironmentVariablesNodeProperty': {
                    'env': na['env']
                }
            })
        if 'tool_location' in na:
            node_props.update({
                "hudson-tools-ToolLocationNodeProperty": {
                    "locations": na['tool_location']
                }
            })

        params = {
            'name': self.name,
            'type': 'hudson.slaves.DumbSlave$DescriptorImpl',
            'json': json.dumps({
                'name': self.name,
                'nodeDescription': na.get('node_description', ''),
                'numExecutors': na['num_executors'],
                'remoteFS': na['remote_fs'],
                'labelString': na['labels'],
                'mode': 'EXCLUSIVE' if na['exclusive'] else 'NORMAL',
                'retentionStrategy': retention,
                'type': 'hudson.slaves.DumbSlave',
                'nodeProperties': node_props,
                'launcher': launcher
            })
        }

        return params

    def get_jenkins_obj(self):
        return self.jenkins

    def __str__(self):
        return self.name

    def is_online(self):
        return not self.poll(tree='offline')['offline']

    def is_temporarily_offline(self):
        return self.poll(tree='temporarilyOffline')['temporarilyOffline']

    def is_jnlpagent(self):
        return self._data['jnlpAgent']

    def is_idle(self):
        return self.poll(tree='idle')['idle']

    def set_online(self):
        """
        Set node online.
        Before change state verify client state: if node set 'offline'
        but 'temporarilyOffline' is not set - client has connection problems
        and AssertionError raised.
        If after run node state has not been changed raise AssertionError.
        """
        self.poll()
        # Before change state check if client is connected
        if self._data['offline'] and not self._data['temporarilyOffline']:
            raise AssertionError("Node is offline and not marked as "
                                 "temporarilyOffline, check client "
                                 "connection: offline = %s, "
                                 "temporarilyOffline = %s" %
                                 (self._data['offline'],
                                  self._data['temporarilyOffline']))

        if self._data['offline'] and self._data['temporarilyOffline']:
            self.toggle_temporarily_offline()
            if self._data['offline']:
                raise AssertionError("The node state is still offline, "
                                     "check client connection:"
                                     " offline = %s, "
                                     "temporarilyOffline = %s" %
                                     (self._data['offline'],
                                      self._data['temporarilyOffline']))

    def set_offline(self, message="requested from jenkinsapi"):
        """
        Set node offline.
        If after run node state has not been changed raise AssertionError.
        : param message: optional string explain why you are taking this
            node offline
        """
        if not self._data['offline']:
            self.toggle_temporarily_offline(message)
            data = self.poll(tree='offline,temporarilyOffline')
            if not data['offline']:
                raise AssertionError("The node state is still online:" +
                                     "offline = %s , temporarilyOffline = %s" %
                                     (data['offline'],
                                      data['temporarilyOffline']))

    def toggle_temporarily_offline(self, message="requested from jenkinsapi"):
        """
        Switches state of connected node (online/offline) and
        set 'temporarilyOffline' property (True/False)
        Calling the same method again will bring node status back.
        :param message: optional string can be used to explain why you
            are taking this node offline
        """
        initial_state = self.is_temporarily_offline()
        url = self.baseurl + \
            "/toggleOffline?offlineMessage=" + urlquote(message)
        try:
            html_result = self.jenkins.requester.get_and_confirm_status(url)
        except PostRequired:
            html_result = self.jenkins.requester.post_and_confirm_status(
                url,
                data={})

        self.poll()
        log.debug(html_result)
        state = self.is_temporarily_offline()
        if initial_state == state:
            raise AssertionError(
                "The node state has not changed: temporarilyOffline = %s" %
                state)

    def update_offline_reason(self, reason):
        """
        Update offline reason on a temporary offline clsuter
        """

        if self.is_temporarily_offline():
            url = self.baseurl + '/changeOfflineCause?offlineMessage=' + urlquote(reason)
            self.jenkins.requester.post_and_confirm_status(url, data={})

    def offline_reason(self):
        return self._data['offlineCauseReason']

    @property
    def _et(self):
        return self._get_config_element_tree()

    def _get_config_element_tree(self):
        """
        Returns an xml element tree for the node's config.xml. The
        resulting tree is cached for quick lookup.
        """
        if self._config is None:
            self.load_config()

        if self._element_tree is None:
            self._element_tree = ET.fromstring(self._config)
        return self._element_tree

    def get_config(self):
        """
        Returns the config.xml from the node.
        """
        response = self.jenkins.requester.get_and_confirm_status(
            "%(baseurl)s/config.xml" % self.__dict__)
        return response.text

    def load_config(self):
        """
        Loads the config.xml for the node allowing it to be re-queried
        without generating new requests.
        """
        if self.name == 'master':
            raise JenkinsAPIException('master node does not have config.xml')

        self._config = self.get_config()
        self._get_config_element_tree()

    def upload_config(self, config_xml):
        """
        Uploads config_xml to the config.xml for the node.
        """
        if self.name == 'master':
            raise JenkinsAPIException('master node does not have config.xml')

        self.jenkins.requester.post_and_confirm_status(
            "%(baseurl)s/config.xml" % self.__dict__,
            data=config_xml)

    def get_labels(self):
        """
        Returns the labels for a slave as a string with each label
        separated by the ' ' character.
        """
        return self.get_config_element('label')

    def get_num_executors(self):
        try:
            return self.get_config_element('numExecutors')
        except JenkinsAPIException:
            return self._data['numExecutors']

    def set_num_executors(self, value):
        """
        Sets number of executors for node

        Warning! Setting number of executors on master node will erase all
        other settings
        """
        set_value = value if isinstance(value, str) else str(value)

        if self.name == 'master':
            # master node doesn't have config.xml, so we're going to submit
            # form here
            data = 'json=%s' % urlquote(
                json.dumps({
                    "numExecutors": set_value,
                    "nodeProperties": {
                        "stapler-class-bag": "true"
                    }
                })
            )

            url = self.baseurl + '/configSubmit'
            self.jenkins.requester.post_and_confirm_status(url, data=data)
        else:
            self.set_config_element('numExecutors', set_value)

        self.poll()

    def get_config_element(self, el_name):
        """
        Returns simple config element.

        Better not to be used to return "nodeProperties" or "launcher"
        """
        return self._et.find(el_name).text

    def set_config_element(self, el_name, value):
        """
        Sets simple config element
        """
        self._et.find(el_name).text = value
        xml_str = ET.tostring(self._et)
        self.upload_config(xml_str)

    def get_monitor(self, monitor_name, poll_monitor=True):
        """
        Polls the node returning one of the monitors in the monitorData branch of the
        returned node api tree.
        """
        monitor_data_key = 'monitorData'
        if poll_monitor:
            # polling as monitors like response time can be updated
            monitor_data = self.poll(tree=monitor_data_key)[monitor_data_key]
        else:
            monitor_data = self._data[monitor_data_key]

        full_monitor_name = 'hudson.node_monitors.{0}'.format(monitor_name)
        if full_monitor_name not in monitor_data:
            raise AssertionError('Node monitor %s not found' % monitor_name)

        return monitor_data[full_monitor_name]

    def get_available_physical_memory(self):
        """
        Returns the node's available physical memory in bytes.
        """
        monitor_data = self.get_monitor('SwapSpaceMonitor')
        return monitor_data['availablePhysicalMemory']

    def get_available_swap_space(self):
        """
        Returns the node's available swap space in bytes.
        """
        monitor_data = self.get_monitor('SwapSpaceMonitor')
        return monitor_data['availableSwapSpace']

    def get_total_physical_memory(self):
        """
        Returns the node's total physical memory in bytes.
        """
        monitor_data = self.get_monitor('SwapSpaceMonitor')
        return monitor_data['totalPhysicalMemory']

    def get_total_swap_space(self):
        """
        Returns the node's total swap space in bytes.
        """
        monitor_data = self.get_monitor('SwapSpaceMonitor')
        return monitor_data['totalSwapSpace']

    def get_workspace_path(self):
        """
        Returns the local path to the node's Jenkins workspace directory.
        """
        monitor_data = self.get_monitor('DiskSpaceMonitor')
        return monitor_data['path']

    def get_workspace_size(self):
        """
        Returns the size in bytes of the node's Jenkins workspace directory.
        """
        monitor_data = self.get_monitor('DiskSpaceMonitor')
        return monitor_data['size']

    def get_temp_path(self):
        """
        Returns the local path to the node's temp directory.
        """
        monitor_data = self.get_monitor('TemporarySpaceMonitor')
        return monitor_data['path']

    def get_temp_size(self):
        """
        Returns the size in bytes of the node's temp directory.
        """
        monitor_data = self.get_monitor('TemporarySpaceMonitor')
        return monitor_data['size']

    def get_architecture(self):
        """
        Returns the system architecture of the node eg. "Linux (amd64)".
        """
        # no need to poll as the architecture will never change
        return self.get_monitor('ArchitectureMonitor', poll_monitor=False)

    def block_until_idle(self, timeout, poll_time=5):
        """
        Blocks until the node become idle.
        :param timeout: Time in second when the wait is aborted.
        :param poll_time: Interval in seconds between each check.
        :@raise TimeOut
        """
        start_time = time.time()
        while not self.is_idle() and (time.time() - start_time) < timeout:
            log.debug(
                "Waiting for the node to become idle. Elapsed time: %s",
                (time.time() - start_time)
            )
            time.sleep(poll_time)

        if not self.is_idle():
            raise TimeOut(
                "The node has not become idle after {} minutes."
                .format(timeout/60)
            )

    def get_response_time(self):
        """
        Returns the node's average response time.
        """
        monitor_data = self.get_monitor('ResponseTimeMonitor')
        return monitor_data['average']

    def get_clock_difference(self):
        """
        Returns the difference between the node's clock and the master Jenkins clock.
        Used to detect out of sync clocks.
        """
        monitor_data = self.get_monitor('ClockMonitor')
        return monitor_data['diff']



"""
Extract version information from the latest build.
"""
from __future__ import print_function
from jenkinsapi.jenkins import Jenkins


job_name = 'foo'
jenkins = Jenkins('http://localhost:8080')
job = jenkins[job_name]
lgb = job.get_last_good_build()
print(lgb.get_revision())



'''
System tests for authentication functionality
'''

import pytest
from jenkinsapi.utils.requester import Requester
from six.moves.urllib.error import HTTPError
from requests import HTTPError as REQHTTPError
from jenkinsapi.jenkins import Jenkins


def test_normal_uthentication(jenkins_admin_admin):
    # No problem with the righ user/pass
    jenkins_user = Jenkins(jenkins_admin_admin.baseurl,
                           jenkins_admin_admin.username,
                           jenkins_admin_admin.password)

    assert jenkins_user is not None

    # We cannot connect if no user/pass
    with pytest.raises(REQHTTPError) as http_excep:
        Jenkins(jenkins_admin_admin.baseurl)

    assert Requester.AUTH_COOKIE is None
    assert http_excep.value.response.status_code == 403


def test_auth_cookie(jenkins_admin_admin):
    initial_cookie_value = None
    final_cookie_value = "JSESSIONID"
    assert initial_cookie_value == Requester.AUTH_COOKIE

    jenkins_admin_admin.use_auth_cookie()
    result = Requester.AUTH_COOKIE
    assert result is not None
    assert final_cookie_value in result


def test_wrongauth_cookie(jenkins_admin_admin):
    initial_cookie_value = None
    assert initial_cookie_value == Requester.AUTH_COOKIE

    jenkins_admin_admin.username = "fakeuser"
    jenkins_admin_admin.password = "fakepass"

    with pytest.raises(HTTPError) as http_excep:
        jenkins_admin_admin.use_auth_cookie()

    assert Requester.AUTH_COOKIE is None
    assert http_excep.value.code == 401


def test_verify_cookie_isworking(jenkins_admin_admin):
    initial_cookie_value = None
    final_cookie_value = "JSESSIONID"
    assert initial_cookie_value == Requester.AUTH_COOKIE

    # Remove requester user/pass
    jenkins_admin_admin.requester.username = None
    jenkins_admin_admin.requester.password = None

    # Verify that we cannot connect
    with pytest.raises(REQHTTPError) as http_excep:
        jenkins_admin_admin.poll()

    assert Requester.AUTH_COOKIE is None
    assert http_excep.value.response.status_code == 403

    # Retrieve the auth cookie, we can because we
    # have right values for jenkins_admin_admin.username
    # and jenkins_admin_admin.password
    jenkins_admin_admin.use_auth_cookie()

    result = Requester.AUTH_COOKIE
    assert result is not None
    assert final_cookie_value in result

    # Verify that we can connect even with no requester user/pass
    # If we have the cookie the requester user/pass is not needed
    jenkins_admin_admin.poll()



'''
System tests for `jenkinsapi.jenkins` module.
'''
import pytest
from jenkinsapi.job import Job
from jenkinsapi.jobs import Jobs
from jenkinsapi.plugin import Plugin
from jenkinsapi.build import Build
from jenkinsapi.queue import QueueItem
from jenkinsapi_tests.systests.job_configs import EMPTY_JOB
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi.custom_exceptions import UnknownJob


def job_present(jenkins, name):
    jenkins.poll()
    assert name in jenkins, 'Job %r is absent in jenkins.' % name
    assert isinstance(jenkins.get_job(name), Job) is True
    assert isinstance(jenkins[name], Job) is True


def job_absent(jenkins, name):
    jenkins.poll()
    assert name not in jenkins, 'Job %r is present in jenkins.' % name


def test_create_job(jenkins):
    job_name = 'create_%s' % random_string()
    jenkins.create_job(job_name, EMPTY_JOB)
    job_present(jenkins, job_name)


def test_create_job_with_plus(jenkins):
    job_name = 'create+%s' % random_string()
    jenkins.create_job(job_name, EMPTY_JOB)
    job_present(jenkins, job_name)
    job = jenkins[job_name]
    assert job_name in job.url


def test_create_dup_job(jenkins):
    job_name = 'create_%s' % random_string()
    old_job = jenkins.create_job(job_name, EMPTY_JOB)
    job_present(jenkins, job_name)
    new_job = jenkins.create_job(job_name, EMPTY_JOB)
    assert new_job == old_job


def test_get_jobs_info(jenkins):
    job_name = 'create_%s' % random_string()
    job = jenkins.create_job(job_name, EMPTY_JOB)

    jobs_info = list(jenkins.get_jobs_info())
    assert len(jobs_info) == 1
    for url, name in jobs_info:
        assert url == job.url
        assert name == job.name


def test_create_job_through_jobs_dict(jenkins):
    job_name = 'create_%s' % random_string()
    jenkins.jobs[job_name] = EMPTY_JOB
    job_present(jenkins, job_name)


def test_enable_disable_job(jenkins):
    job_name = 'create_%s' % random_string()
    jenkins.create_job(job_name, EMPTY_JOB)
    job_present(jenkins, job_name)

    j = jenkins[job_name]
    j.invoke(block=True)  # run this at least once

    j.disable()
    assert j.is_enabled() is False, 'A disabled job is reporting incorrectly'

    j.enable()
    assert j.is_enabled() is True, 'An enabled job is reporting incorrectly'


def test_get_job_and_update_config(jenkins):
    job_name = 'config_%s' % random_string()
    jenkins.create_job(job_name, EMPTY_JOB)
    job_present(jenkins, job_name)
    config = jenkins[job_name].get_config()
    assert config.strip() == EMPTY_JOB.strip()
    jenkins[job_name].update_config(EMPTY_JOB)


def test_invoke_job(jenkins):
    job_name = 'create_%s' % random_string()
    job = jenkins.create_job(job_name, EMPTY_JOB)
    job.invoke(block=True)
    assert isinstance(job.get_build(1), Build)


def test_invocation_object(jenkins):
    job_name = 'create_%s' % random_string()
    job = jenkins.create_job(job_name, EMPTY_JOB)
    ii = job.invoke()
    assert isinstance(ii, QueueItem) is True


def test_get_jobs_list(jenkins):
    job1_name = 'first_%s' % random_string()
    job2_name = 'second_%s' % random_string()

    jenkins.create_job(job1_name, EMPTY_JOB)
    jenkins.create_job(job2_name, EMPTY_JOB)
    assert len(jenkins.jobs) >= 2
    job_list = jenkins.get_jobs_list()
    assert [job1_name, job2_name] == job_list


def test_get_job(jenkins):
    job1_name = 'first_%s' % random_string()

    jenkins.create_job(job1_name, EMPTY_JOB)
    job = jenkins[job1_name]
    assert isinstance(job, Job) is True
    assert job.name == job1_name


def test_get_jobs(jenkins):
    job1_name = 'first_%s' % random_string()
    job2_name = 'second_%s' % random_string()

    jenkins.create_job(job1_name, EMPTY_JOB)
    jenkins.create_job(job2_name, EMPTY_JOB)
    jobs = jenkins.jobs
    assert isinstance(jobs, Jobs) is True
    assert len(jobs) >= 2
    for job_name, job in jobs.iteritems():
        assert isinstance(job_name, str) is True
        assert isinstance(job, Job) is True


def test_get_job_that_does_not_exist(jenkins):
    with pytest.raises(UnknownJob):
        jenkins['doesnot_exist']


def test_has_job(jenkins):
    job1_name = 'first_%s' % random_string()
    jenkins.create_job(job1_name, EMPTY_JOB)
    assert jenkins.has_job(job1_name) is True
    assert job1_name in jenkins


def test_has_no_job(jenkins):
    assert jenkins.has_job('doesnt_exist') is False
    assert 'doesnt_exist' not in jenkins


def test_delete_job(jenkins):
    job1_name = 'delete_me_%s' % random_string()

    jenkins.create_job(job1_name, EMPTY_JOB)
    jenkins.delete_job(job1_name)
    job_absent(jenkins, job1_name)


def test_rename_job(jenkins):
    job1_name = 'A__%s' % random_string()
    job2_name = 'B__%s' % random_string()

    jenkins.create_job(job1_name, EMPTY_JOB)
    jenkins.rename_job(job1_name, job2_name)
    job_absent(jenkins, job1_name)
    job_present(jenkins, job2_name)


def test_copy_job(jenkins):
    template_job_name = 'TPL%s' % random_string()
    copied_job_name = 'CPY%s' % random_string()

    jenkins.create_job(template_job_name, EMPTY_JOB)
    j = jenkins.copy_job(template_job_name, copied_job_name)
    job_present(jenkins, template_job_name)
    job_present(jenkins, copied_job_name)
    assert isinstance(j, Job) is True
    assert j.name == copied_job_name


def test_get_master_data(jenkins):
    master_data = jenkins.get_master_data()
    assert master_data['totalExecutors'] == 2


def test_get_missing_plugin(jenkins):
    plugins = jenkins.get_plugins()
    with pytest.raises(KeyError):
        plugins["lsdajdaslkjdlkasj"]  # this plugin surely does not exist!


def test_get_single_plugin(jenkins):
    plugins = jenkins.get_plugins()
    plugin_name, plugin = next(plugins.iteritems())
    assert isinstance(plugin_name, str) is True
    assert isinstance(plugin, Plugin) is True


def test_get_single_plugin_depth_2(jenkins):
    plugins = jenkins.get_plugins(depth=2)
    _, plugin = next(plugins.iteritems())


def test_install_delete_single_plugin_string(jenkins):
    plugin_name = 'simple-theme-plugin'
    plugin_version = 'latest'
    plugin = ('%s@%s') % (plugin_name, plugin_version)

    assert plugin_name not in jenkins.plugins

    jenkins.install_plugin(plugin)

    assert plugin_name in jenkins.plugins

    # Force restart so plugin is deleted from plugin-list
    jenkins.delete_plugin(plugin_name, force_restart=True, wait_for_reboot=True)

    assert plugin_name not in jenkins.plugins


def test_install_delete_single_plugin_object(jenkins):
    plugin_name = 'simple-theme-plugin'
    plugin_version = 'latest'
    plugin = Plugin(('%s@%s') % (plugin_name, plugin_version))

    assert plugin_name not in jenkins.plugins

    jenkins.install_plugin(plugin, restart=True, wait_for_reboot=True)

    assert plugin_name in jenkins.plugins

    # Force restart so plugin is deleted from plugin-list
    jenkins.delete_plugin(plugin, force_restart=True, wait_for_reboot=True)

    assert plugin_name not in jenkins.plugins


def test_install_delete_multiple_plugins_mix_string_object(jenkins):
    plugin_one_name = 'simple-theme-plugin'
    plugin_one_version = 'latest'
    plugin_one = ('%s@%s') % (plugin_one_name, plugin_one_version)
    plugin_two_name = 'docker-commons'
    plugin_two_version = 'latest'
    plugin_two = Plugin(('%s@%s') % (plugin_two_name, plugin_two_version))

    assert (plugin_one_name and plugin_two_name) not in jenkins.plugins

    jenkins.install_plugins([plugin_one, plugin_two], restart=True,
                            wait_for_reboot=True)

    assert (plugin_one_name and plugin_two_name) in jenkins.plugins

    # Force restart so plugins are deleted from plugin-list
    jenkins.delete_plugins([plugin_one_name, plugin_two], force_restart=True,
                           wait_for_reboot=True)

    assert (plugin_one_name and plugin_two_name) not in jenkins.plugins


def test_run_groovy_script(jenkins):
    expected_result = 'Hello world!'
    result = jenkins.run_groovy_script('print "%s"' % expected_result)
    assert result.strip() == 'Hello world!'



import pytest
from collections import namedtuple

import jenkinsapi
from jenkinsapi.plugins import Plugins
from jenkinsapi.utils.requester import Requester
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.job import Job
from jenkinsapi.custom_exceptions import JenkinsAPIException

DATA = {}
TWO_JOBS_DATA = {
    'jobs': [
        {'name': 'job_one',
         'url': 'http://localhost:8080/job/job_one',
         'color': 'blue'},
        {'name': 'job_two',
         'url': 'http://localhost:8080/job/job_two',
         'color': 'blue'},
    ]
}
MULTIBRANCH_JOBS_DATA = {
    'jobs': [
        {'name': 'multibranch-repo/master',
         'url': 'http://localhost:8080/job/multibranch-repo/job/master',
         'color': 'blue'},
        {'name': 'multibranch-repo/develop',
         'url': 'http://localhost:8080/job/multibranch-repo/job/develop',
         'color': 'blue'},
    ]
}
SCAN_MULTIBRANCH_PIPELINE_LOG = """
Started by timer
[Fri Jul 05 06:46:00 CEST 2019] Starting branch indexing...
Connecting to https://stash.macq.eu using Jenkins/****** (jenkins-ldap)
Repository type: Git
Looking up internal/base for branches
Checking branch master from internal/base
      'Jenkinsfile' found
    Met criteria
No changes detected: master (still at 26d4d8a673f57a957fd5a23f5adfe0be02089294)

  1 branches were processed
Looking up internal/base for pull requests

  0 pull requests were processed
[Fri Jul 05 06:46:01 CEST 2019] Finished branch indexing. Indexing took 1.1 sec
Finished: SUCCESS
"""


@pytest.fixture(scope='function')
def jenkins(monkeypatch):
    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, '_poll', fake_poll)

    return Jenkins('http://localhost:8080',
                   username='foouser', password='foopassword')


def test__clone(jenkins):
    cloned = jenkins._clone()
    assert id(cloned) != id(jenkins)
    assert cloned == jenkins


def test_stored_passwords(jenkins):
    assert jenkins.requester.password == 'foopassword'
    assert jenkins.requester.username == 'foouser'


def test_reload(monkeypatch):
    class FakeResponse(object):
        status_code = 200
        text = '{}'

    def fake_get_url(
            url,  # pylint: disable=unused-argument
            params=None,  # pylint: disable=unused-argument
            headers=None,  # pylint: disable=unused-argument
            allow_redirects=True,  # pylint: disable=unused-argument
            stream=False):  # pylint: disable=unused-argument

        return FakeResponse()

    monkeypatch.setattr(Requester, 'get_url', fake_get_url)
    mock_requester = Requester(username='foouser', password='foopassword')
    jenkins = Jenkins(
        'http://localhost:8080/',
        username='foouser', password='foopassword',
        requester=mock_requester)
    jenkins.poll()


def test_get_jobs_list(monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        return TWO_JOBS_DATA

    def fake_job_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, '_poll', fake_jenkins_poll)
    monkeypatch.setattr(Jenkins, '_poll', fake_jenkins_poll)
    monkeypatch.setattr(Job, '_poll', fake_job_poll)

    jenkins = Jenkins('http://localhost:8080/',
                      username='foouser', password='foopassword')
    for idx, job_name in enumerate(jenkins.get_jobs_list()):
        assert job_name == TWO_JOBS_DATA['jobs'][idx]['name']

    for idx, job_name in enumerate(jenkins.jobs.keys()):
        assert job_name == TWO_JOBS_DATA['jobs'][idx]['name']


def test_create_new_job_fail(mocker, monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        return TWO_JOBS_DATA

    def fake_job_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, '_poll', fake_jenkins_poll)
    monkeypatch.setattr(Jenkins, '_poll', fake_jenkins_poll)
    monkeypatch.setattr(Job, '_poll', fake_job_poll)

    mock_requester = Requester(username='foouser', password='foopassword')
    mock_requester.post_xml_and_confirm_status = mocker.MagicMock(
        return_value=''
    )

    jenkins = Jenkins('http://localhost:8080/',
                      username='foouser', password='foopassword',
                      requester=mock_requester)

    with pytest.raises(JenkinsAPIException) as ar:
        jenkins.create_job('job_new', None)

    assert 'Job XML config cannot be empty' in str(ar.value)


def test_create_multibranch_pipeline_job(mocker, monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        # return multibranch jobs and other jobs.
        # create_multibranch_pipeline_job is supposed to filter out the MULTIBRANCH jobs
        return {
            'jobs': TWO_JOBS_DATA['jobs'] + MULTIBRANCH_JOBS_DATA['jobs']
        }

    def fake_job_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, '_poll', fake_jenkins_poll)
    monkeypatch.setattr(Jenkins, '_poll', fake_jenkins_poll)
    monkeypatch.setattr(Job, '_poll', fake_job_poll)

    mock_requester = Requester(username='foouser', password='foopassword')
    mock_requester.post_xml_and_confirm_status = mocker.MagicMock(
        return_value=''
    )
    mock_requester.post_and_confirm_status = mocker.MagicMock(
        return_value=''
    )
    get_response = namedtuple('get_response', 'text')
    mock_requester.get_url = mocker.MagicMock(
        return_value=get_response(text=SCAN_MULTIBRANCH_PIPELINE_LOG)
    )
    jenkins = Jenkins('http://localhost:8080/',
                      username='foouser', password='foopassword',
                      requester=mock_requester)

    jobs = jenkins.create_multibranch_pipeline_job("multibranch-repo", "multibranch-xml-content")

    for idx, job_instance in enumerate(jobs):
        assert job_instance.name == MULTIBRANCH_JOBS_DATA['jobs'][idx]['name']

    # make sure we didn't get more jobs.
    assert len(MULTIBRANCH_JOBS_DATA['jobs']) == len(jobs)


def test_get_jenkins_obj_from_url(mocker, monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        return TWO_JOBS_DATA

    def fake_job_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, '_poll', fake_jenkins_poll)
    monkeypatch.setattr(Jenkins, '_poll', fake_jenkins_poll)
    monkeypatch.setattr(Job, '_poll', fake_job_poll)

    mock_requester = Requester(username='foouser', password='foopassword')
    mock_requester.post_xml_and_confirm_status = mocker.MagicMock(
        return_value=''
    )

    jenkins = Jenkins('http://localhost:8080/',
                      username='foouser', password='foopassword',
                      requester=mock_requester)

    new_jenkins = jenkins.get_jenkins_obj_from_url('http://localhost:8080/')
    assert new_jenkins == jenkins

    new_jenkins = jenkins.get_jenkins_obj_from_url('http://localhost:8080/foo')
    assert new_jenkins != jenkins


def test_get_jenkins_obj(mocker, monkeypatch):
    def fake_jenkins_poll(cls, tree=None):  # pylint: disable=unused-argument
        return TWO_JOBS_DATA

    def fake_job_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, '_poll', fake_jenkins_poll)
    monkeypatch.setattr(Jenkins, '_poll', fake_jenkins_poll)
    monkeypatch.setattr(Job, '_poll', fake_job_poll)

    mock_requester = Requester(username='foouser', password='foopassword')
    mock_requester.post_xml_and_confirm_status = mocker.MagicMock(
        return_value=''
    )

    jenkins = Jenkins('http://localhost:8080/',
                      username='foouser', password='foopassword',
                      requester=mock_requester)

    new_jenkins = jenkins.get_jenkins_obj()
    assert new_jenkins == jenkins


def test_get_version(monkeypatch):
    class MockResponse(object):
        def __init__(self):
            self.headers = {}
            self.headers['X-Jenkins'] = '1.542'

    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, '_poll', fake_poll)

    def fake_get(cls, *arga, **kwargs):  # pylint: disable=unused-argument
        return MockResponse()

    monkeypatch.setattr(Requester, 'get_and_confirm_status', fake_get)

    jenkins = Jenkins('http://foobar:8080/',
                      username='foouser', password='foopassword')
    assert jenkins.version == '1.542'


def test_get_version_nonexistent(mocker):
    class MockResponse(object):
        status_code = 200
        headers = {}
        text = '{}'

    mock_requester = Requester(username='foouser', password='foopassword')
    mock_requester.get_url = mocker.MagicMock(
        return_value=MockResponse()
    )
    jenkins = Jenkins('http://localhost:8080',
                      username='foouser', password='foopassword',
                      requester=mock_requester)
    assert jenkins.version == '0.0'


def test_get_master_data(mocker):
    class MockResponse(object):
        status_code = 200
        headers = {}
        text = '{}'

    mock_requester = Requester(username='foouser', password='foopassword')
    mock_requester.get_url = mocker.MagicMock(
        return_value=MockResponse()
    )
    jenkins = Jenkins('http://localhost:808',
                      username='foouser', password='foopassword',
                      requester=mock_requester)
    jenkins.get_data = mocker.MagicMock(
        return_value={
            "busyExecutors": 59,
            "totalExecutors": 75
        }
    )

    data = jenkins.get_master_data()
    assert data['busyExecutors'] == 59
    assert data['totalExecutors'] == 75


def test_get_create_url(monkeypatch):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, '_poll', fake_poll)
    # Jenkins URL w/o slash
    jenkins = Jenkins('http://localhost:8080',
                      username='foouser', password='foopassword')
    assert jenkins.get_create_url() == 'http://localhost:8080/createItem'
    # Jenkins URL w/ slash
    jenkins = Jenkins('http://localhost:8080/',
                      username='foouser', password='foopassword')
    assert jenkins.get_create_url() == 'http://localhost:8080/createItem'


def test_has_plugin(monkeypatch):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, '_poll', fake_poll)

    def fake_plugin_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {
            'plugins': [
                {
                    'deleted': False, 'hasUpdate': True, 'downgradable': False,
                    'dependencies': [{}, {}, {}, {}],
                    'longName': 'Jenkins Subversion Plug-in', 'active': True,
                    'shortName': 'subversion', 'backupVersion': None,
                    'url': 'http://wiki.jenkins-ci.org/'
                           'display/JENKINS/Subversion+Plugin',
                    'enabled': True, 'pinned': False, 'version': '1.45',
                    'supportsDynamicLoad': 'MAYBE', 'bundled': True
                }
            ]
        }

    monkeypatch.setattr(Plugins, '_poll', fake_plugin_poll)

    jenkins = Jenkins('http://localhost:8080/',
                      username='foouser', password='foopassword')
    assert jenkins.has_plugin('subversion') is True


def test_get_use_auth_cookie(mocker, monkeypatch):
    COOKIE_VALUE = 'FAKE_COOKIE'

    def fake_opener(redirect_handler):  # pylint: disable=unused-argument
        mock_response = mocker.MagicMock()
        mock_response.cookie = COOKIE_VALUE

        mock_opener = mocker.MagicMock()
        mock_opener.open.return_value = mock_response
        return mock_opener

    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, '_poll', fake_poll)
    monkeypatch.setattr(Requester, 'AUTH_COOKIE', None)
    monkeypatch.setattr(jenkinsapi.jenkins, 'build_opener', fake_opener)

    jenkins = Jenkins('http://localhost:8080',
                      username='foouser', password='foopassword')

    jenkins.use_auth_cookie()
    assert Requester.AUTH_COOKIE == COOKIE_VALUE



"""
Get information about currently installed plugins
"""
from __future__ import print_function
from jenkinsapi.jenkins import Jenkins


plugin_name = 'subversion'
jenkins = Jenkins('http://localhost:8080')
plugin = jenkins.get_plugins()[plugin_name]

print(repr(plugin))



"""
Module for jenkinsapi Plugin
"""


class Plugin(object):

    """
    Plugin class
    """

    def __init__(self, plugin_dict):
        if isinstance(plugin_dict, dict):
            self.__dict__ = plugin_dict
        else:
            self.__dict__ = self.to_plugin(plugin_dict)

    def to_plugin(self, plugin_string):
        plugin_string = str(plugin_string)
        if '@' not in plugin_string or len(plugin_string.split('@')) != 2:
            usage_err = ('plugin specification must be a string like '
                         '"plugin-name@version", not "{0}"')
            usage_err = usage_err.format(plugin_string)
            raise ValueError(usage_err)

        shortName, version = plugin_string.split('@')
        return {'shortName': shortName, 'version': version}

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __str__(self):
        return self.shortName

    def __repr__(self):
        return "<%s.%s %s>" % (
            self.__class__.__module__,
            self.__class__.__name__,
            str(self)
        )

    def get_attributes(self):
        """
        Used by Plugins object to install plugins in Jenkins
        """
        return (
            "<jenkins> <install plugin=\"%s@%s\" /> </jenkins>"
            % (self.shortName, self.version)
        )

    def is_latest(self, update_center_dict):
        """
        Used by Plugins object to determine if plugin can be
        installed through the update center (when plugin version is
        latest version), or must be installed by uploading
        the plugin hpi file.
        """
        if self.version == 'latest':
            return True
        center_plugin = update_center_dict['plugins'][self.shortName]
        return center_plugin['version'] == self.version

    def get_download_link(self, update_center_dict):
        latest_version = update_center_dict[
            'plugins'][self.shortName]['version']
        latest_url = update_center_dict['plugins'][self.shortName]['url']
        return latest_url.replace(
            "/".join(
                (self.shortName, latest_version)),
            "/".join((self.shortName, self.version)))



import os
import time
import shutil
import logging
import datetime
import tempfile
import posixpath
import requests
import threading
import subprocess
from pkg_resources import resource_stream
from tarfile import TarFile
from six.moves import queue
from six.moves.urllib.parse import urlparse

from jenkinsapi.jenkins import Jenkins
from jenkinsapi.custom_exceptions import JenkinsAPIException

log = logging.getLogger(__name__)


class FailedToStart(Exception):
    pass


class TimeOut(Exception):
    pass


class StreamThread(threading.Thread):

    def __init__(self, name, q, stream, fn_log):
        threading.Thread.__init__(self)
        self.name = name
        self.queue = q
        self.stream = stream
        self.fn_log = fn_log
        self._stop = threading.Event()

    def stop(self):
        self._stop.set()

    def stopped(self):
        return self._stop.isSet()

    def run(self):
        log.info("Starting %s", self.name)

        while True:
            if self._stop.isSet():
                break
            line = self.stream.readline()
            if line:
                self.fn_log(line.rstrip())
                self.queue.put((self.name, line))
            else:
                break

        self.queue.put((self.name, None))


class JenkinsLancher(object):

    """
    Launch jenkins
    """
    JENKINS_WEEKLY_WAR_URL = (
        "http://updates.jenkins.io/latest/jenkins.war"
    )
    JENKINS_LTS_WAR_URL = (
        "https://updates.jenkins.io/stable/latest/jenkins.war"
    )

    def __init__(self, local_orig_dir, systests_dir, war_name, plugin_urls=None,
                 jenkins_url=None):
        if jenkins_url is not None:
            self.jenkins_url = jenkins_url
            self.http_port = urlparse(jenkins_url).port
            self.start_new_instance = False
        else:
            import socket
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.bind(('', 0))
            sock.listen(1)
            port = sock.getsockname()[1]
            sock.close()
            self.http_port = port
            self.jenkins_url = 'http://localhost:%s' % self.http_port
            self.start_new_instance = True

        self.threads = []
        self.war_path = os.path.join(local_orig_dir, war_name)
        self.local_orig_dir = local_orig_dir
        self.systests_dir = systests_dir
        self.war_filename = war_name

        if 'JENKINS_HOME' not in os.environ:
            self.jenkins_home = tempfile.mkdtemp(prefix='jenkins-home-')
            os.environ['JENKINS_HOME'] = self.jenkins_home
        else:
            self.jenkins_home = os.environ['JENKINS_HOME']

        self.jenkins_process = None
        self.queue = queue.Queue()
        self.plugin_urls = plugin_urls or []
        if os.environ.get('JENKINS_VERSION', 'stable') == 'stable':
            self.JENKINS_WAR_URL = self.JENKINS_LTS_WAR_URL
        else:
            self.JENKINS_WAR_URL = self.JENKINS_WEEKLY_WAR_URL

    def update_war(self):
        os.chdir(self.systests_dir)
        if os.path.exists(self.war_path):
            log.info("War file already present, delete it to redownload and"
                     " update jenkins")
        else:
            log.info("Downloading Jenkins War")
            script_dir = os.path.join(self.systests_dir,
                                      'get-jenkins-war.sh')
            subprocess.check_call([script_dir,
                                   self.JENKINS_WAR_URL, self.local_orig_dir,
                                   self.war_filename])

    def update_config(self):
        tarball = TarFile.open(fileobj=resource_stream(
            'jenkinsapi_tests.systests', 'jenkins_home.tar.gz'))
        tarball.extractall(path=self.jenkins_home)

    def install_plugins(self):
        plugin_dest_dir = os.path.join(self.jenkins_home, 'plugins')
        log.info("Plugins will be installed in '%s'", plugin_dest_dir)

        if not os.path.exists(plugin_dest_dir):
            os.mkdir(plugin_dest_dir)

        for url in self.plugin_urls:
            self.install_plugin(url, plugin_dest_dir)

    def install_plugin(self, hpi_url, plugin_dest_dir):
        path = urlparse(hpi_url).path
        filename = posixpath.basename(path)
        plugin_orig_dir = os.path.join(self.local_orig_dir, 'plugins')
        if not os.path.exists(plugin_orig_dir):
            os.mkdir(plugin_orig_dir)
        plugin_orig_path = os.path.join(plugin_orig_dir, filename)
        plugin_dest_path = os.path.join(plugin_dest_dir, filename)
        if os.path.exists(plugin_orig_path):
            log.info("%s already locally present, delete the file to redownload"
                     " and update", filename)
        else:
            log.info("Downloading %s from %s", filename, hpi_url)
            with open(plugin_orig_path, 'wb') as hpi:
                request = requests.get(hpi_url)
                hpi.write(request.content)
        log.info("Installing %s", filename)
        shutil.copy(plugin_orig_path, plugin_dest_path)
        # Create an empty .pinned file, so that the downloaded plugin
        # will be used, instead of the version bundled in jenkins.war
        # See https://wiki.jenkins-ci.org/display/JENKINS/Pinned+Plugins
        open(plugin_dest_path + ".pinned", 'a').close()

    def stop(self):
        if self.start_new_instance:
            log.info("Shutting down jenkins.")
            # Start the threads
            for thread in self.threads:
                thread.stop()

            Jenkins(self.jenkins_url).shutdown()
            # self.jenkins_process.terminate()
            # self.jenkins_process.wait()
            # Do not remove jenkins home if JENKINS_URL is set
            if 'JENKINS_URL' not in os.environ:
                shutil.rmtree(self.jenkins_home, ignore_errors=True)
            log.info("Jenkins stopped.")

    def block_until_jenkins_ready(self, timeout):
        start_time = datetime.datetime.now()
        timeout_time = start_time + datetime.timedelta(seconds=timeout)

        while True:
            try:
                Jenkins(self.jenkins_url)
                log.info('Jenkins is finally ready for use.')
            except JenkinsAPIException:
                log.info('Jenkins is not yet ready...')
            if datetime.datetime.now() > timeout_time:
                raise TimeOut('Took too long for Jenkins to become ready...')
            time.sleep(5)

    def start(self, timeout=60):
        if self.start_new_instance:
            self.jenkins_home = os.environ.get('JENKINS_HOME',
                                               self.jenkins_home)
            self.update_war()
            self.update_config()
            self.install_plugins()

            os.chdir(self.local_orig_dir)

            jenkins_command = ['java',
                               '-Djenkins.install.runSetupWizard=false',
                               '-Dhudson.DNSMultiCast.disabled=true',
                               '-jar', self.war_filename,
                               '--httpPort=%d' % self.http_port]

            log.info("About to start Jenkins...")
            log.info("%s> %s", os.getcwd(), " ".join(jenkins_command))
            self.jenkins_process = subprocess.Popen(
                jenkins_command, stdin=subprocess.PIPE,
                stdout=subprocess.PIPE, stderr=subprocess.PIPE)

            self.threads = [
                StreamThread('out', self.queue, self.jenkins_process.stdout,
                             log.info),
                StreamThread('err', self.queue, self.jenkins_process.stderr,
                             log.warning)
            ]

            # Start the threads
            for thread in self.threads:
                thread.start()

            while True:
                try:
                    streamName, line = self.queue.get(
                        block=True, timeout=timeout)
                    # Python 3.x
                    if isinstance(line, bytes):
                        line = line.decode('UTF-8')
                except queue.Empty:
                    log.warning("Input ended unexpectedly")
                    break
                else:
                    if line:
                        if 'Failed to initialize Jenkins' in line:
                            raise FailedToStart(line)

                        if 'Invalid or corrupt jarfile' in line:
                            raise FailedToStart(line)

                        if 'is fully up and running' in line:
                            log.info(line)
                            return
                    else:
                        log.warning('Stream %s has terminated', streamName)

            self.block_until_jenkins_ready(timeout)


if __name__ == '__main__':
    logging.basicConfig()
    logging.getLogger('').setLevel(logging.INFO)

    log.info("Hello!")

    jl = JenkinsLancher(
        '/home/sal/workspace/jenkinsapi/src/'
        'jenkinsapi_tests/systests/',
        'jenkins.war'
    )
    jl.start()
    log.info("Jenkins was launched...")

    time.sleep(30)

    log.info("...now to shut it down!")
    jl.stop()



from __future__ import print_function

try:
    from SimpleHTTPServer import SimpleHTTPRequestHandler
except ImportError:
    from http.server import SimpleHTTPRequestHandler

try:
    import SocketServer as socketserver
except ImportError:
    import socketserver

import logging
import cgi

PORT = 8080


class ServerHandler(SimpleHTTPRequestHandler):

    def do_GET(self):
        logging.error(self.headers)
        super().do_GET()

    def do_POST(self):
        logging.error(self.headers)
        form = cgi.FieldStorage(
            fp=self.rfile,
            headers=self.headers,
            environ={'REQUEST_METHOD': 'POST',
                     'CONTENT_TYPE': self.headers['Content-Type'],
                     })
        for item in form.list:
            logging.error(item)
        super().do_GET()


Handler = ServerHandler

httpd = socketserver.TCPServer(("", PORT), Handler)

print("serving at port", PORT)
httpd.serve_forever()



"""
System tests for `jenkinsapi.jenkins` module.
"""
import time
import logging
import pytest
from jenkinsapi.build import Build
from jenkinsapi.queue import QueueItem
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi_tests.systests.job_configs import LONG_RUNNING_JOB
from jenkinsapi_tests.systests.job_configs import SHORTISH_JOB, EMPTY_JOB
from jenkinsapi.custom_exceptions import BadParams, NotFound


log = logging.getLogger(__name__)


def test_invocation_object(jenkins):
    job_name = 'Acreate_%s' % random_string()
    job = jenkins.create_job(job_name, SHORTISH_JOB)
    qq = job.invoke()
    assert isinstance(qq, QueueItem)
    # Let Jenkins catchup
    qq.block_until_building()
    assert qq.get_build_number() == 1


def test_get_block_until_build_running(jenkins):
    job_name = 'Bcreate_%s' % random_string()
    job = jenkins.create_job(job_name, LONG_RUNNING_JOB)
    qq = job.invoke()
    time.sleep(3)
    bn = qq.block_until_building(delay=3).get_number()
    assert isinstance(bn, int)

    build = qq.get_build()
    assert isinstance(build, Build)
    assert build.is_running()
    build.stop()
    # if we call next line right away - Jenkins have no time to stop job
    # so we wait a bit
    time.sleep(1)
    assert not build.is_running()
    console = build.get_console()
    assert isinstance(console, str)
    assert 'Started by user' in console


def test_get_block_until_build_complete(jenkins):
    job_name = 'Ccreate_%s' % random_string()
    job = jenkins.create_job(job_name, SHORTISH_JOB)
    qq = job.invoke()
    qq.block_until_complete()
    assert not qq.get_build().is_running()


def test_mi_and_get_last_build(jenkins):
    job_name = 'Dcreate_%s' % random_string()

    job = jenkins.create_job(job_name, SHORTISH_JOB)

    for _ in range(3):
        ii = job.invoke()
        ii.block_until_complete(delay=2)

    build_number = job.get_last_good_buildnumber()
    assert build_number == 3

    build = job.get_build(build_number)
    assert isinstance(build, Build)

    build = job.get_build_metadata(build_number)
    assert isinstance(build, Build)


def test_mi_and_get_build_number(jenkins):
    job_name = 'Ecreate_%s' % random_string()

    job = jenkins.create_job(job_name, EMPTY_JOB)

    for invocation in range(3):
        qq = job.invoke()
        qq.block_until_complete(delay=1)
        build_number = qq.get_build_number()
        assert build_number == invocation + 1


def test_mi_and_delete_build(jenkins):
    job_name = 'Ecreate_%s' % random_string()

    job = jenkins.create_job(job_name, EMPTY_JOB)

    for invocation in range(3):
        qq = job.invoke()
        qq.block_until_complete(delay=1)
        build_number = qq.get_build_number()
        assert build_number == invocation + 1

    # Delete build using Job.delete_build
    job.get_build(1)
    job.delete_build(1)
    with pytest.raises(NotFound):
        job.get_build(1)

    # Delete build using Job as dictionary of builds
    assert isinstance(job[2], Build)
    del job[2]

    with pytest.raises(NotFound):
        job.get_build(2)

    with pytest.raises(NotFound):
        job.delete_build(99)


def test_give_params_on_non_parameterized_job(jenkins):
    job_name = 'Ecreate_%s' % random_string()
    job = jenkins.create_job(job_name, EMPTY_JOB)
    with pytest.raises(BadParams):
        job.invoke(build_params={'foo': 'bar', 'baz': 99})


def test_keep_build_toggle(jenkins):
    job_name = 'Ecreate_%s' % random_string()

    job = jenkins.create_job(job_name, EMPTY_JOB)

    qq = job.invoke()
    qq.block_until_complete(delay=1)

    build = job.get_last_build()
    assert not build.is_kept_forever()
    build.toggle_keep()
    assert build.is_kept_forever()

    build_number = job.get_last_buildnumber()
    job.toggle_keep_build(build_number)
    build = job.get_last_build()
    assert not build.is_kept_forever()
